@@ -2,6 +2,7 @@
 #define __SYS_CONFIG_H__
 
 #include <machine/ieeefp.h>  /* floating point macros */
+#include <sys/features.h>	/* POSIX defs */
 
 /* exceptions first */
 #if defined(__H8500__) || defined(__W65__)
@@ -20,7 +21,7 @@
 #define UINT_MAX 65535
 #endif
 
-#if defined (__H8300__) || defined (__H8300H__) || defined(__H8300S__)
+#if defined (__H8300__) || defined (__H8300H__) || defined(__H8300S__) || defined (__H8300SX__)
 #define __SMALL_BITFIELDS
 #define H8300 1
 #undef INT_MAX
@@ -29,6 +30,10 @@
 #define UINT_MAX (__INT_MAX__ * 2U + 1)
 #endif
 
+#if defined (__xc16x__) || defined (__xc16xL__) || defined (__xc16xS__)
+#define __SMALL_BITFIELDS
+#endif
+
 #ifdef __W65__
 #define __SMALL_BITFIELDS
 #endif
@@ -59,7 +64,7 @@
 /* in other words, go32 */
 #define _FLOAT_RET double
 #endif
-#ifdef __linux__
+#if defined(__linux__) || defined(__RDOS__)
 /* we want the reentrancy structure to be returned by a function */
 #define __DYNAMIC_REENT__
 #define HAVE_GETDATE
@@ -94,6 +99,21 @@
 #endif
 #endif
 
+/* Configure small REENT structure for Xilinx MicroBlaze platforms */
+#if defined (__MICROBLAZE__)
+#ifndef _REENT_SMALL
+#define _REENT_SMALL
+#endif
+/* Xilinx XMK uses Unix98 mutex */
+#ifdef __XMK__
+#define _UNIX98_THREAD_MUTEX_ATTRIBUTES
+#endif
+#endif
+
+#if defined(__mips__) && !defined(__rtems__)
+#define __ATTRIBUTE_IMPURE_PTR__ __attribute__((__section__(".sdata")))
+#endif
+
 #ifdef __xstormy16__
 #define __SMALL_BITFIELDS
 #undef INT_MAX
@@ -105,6 +125,26 @@
 #define __BUFSIZ__ 16
 #define _REENT_SMALL
 #endif
+#ifdef __m32c__
+#define __SMALL_BITFIELDS
+#undef INT_MAX
+#undef UINT_MAX
+#define INT_MAX __INT_MAX__
+#define UINT_MAX (__INT_MAX__ * 2U + 1)
+#define MALLOC_ALIGNMENT 8
+#if defined(__r8c_cpu__) || defined(__m16c_cpu__)
+#define _POINTER_INT short
+#else
+#define _POINTER_INT long
+#endif
+#define __BUFSIZ__ 16
+#define _REENT_SMALL
+#endif /* __m32c__ */
+
+#ifdef __SPU__
+#define MALLOC_ALIGNMENT 16
+#define __CUSTOM_FILE_IO__
+#endif
 
 /* This block should be kept in sync with GCC's limits.h.  The point
    of having these definitions here is to not include limits.h, which
@@ -147,15 +187,10 @@
 #define __RAND_MAX 0x7fffffff
 #endif
 
-
-#if defined(__CYGWIN32__) || defined(__CYGWIN__)
-#define __FILENAME_MAX__ (260 - 1 /* NUL */)
-#define _READ_WRITE_RETURN_TYPE _ssize_t
-#define __LARGE64_FILES 1
-#if defined(__INSIDE_CYGWIN__) || defined(_COMPILING_NEWLIB)
-#define __IMPORT
-#else
-#define __IMPORT __declspec(dllimport)
+#if defined(__CYGWIN__)
+#include <cygwin/config.h>
+#if !defined (__STRICT_ANSI__) || (__STDC_VERSION__ >= 199901L)
+#define __USE_XOPEN2K 1
 #endif
 #endif
 
@@ -164,6 +199,10 @@
 #define _READ_WRITE_RETURN_TYPE _ssize_t
 #endif
 
+#ifndef __EXPORT
+#define __EXPORT
+#endif
+
 #ifndef __IMPORT
 #define __IMPORT
 #endif
@@ -176,10 +215,25 @@
 #endif
 
 #ifndef __WCHAR_MAX__
-#if __INT_MAX__ == 32767 || defined(__CYGWIN__) || \
-    defined (_WIN32)
+#if __INT_MAX__ == 32767 || defined (_WIN32)
 #define __WCHAR_MAX__ 0xffffu
 #endif
 #endif
 
+/* See if small reent asked for at configuration time and
+   is not chosen by the platform by default.  */
+#ifdef _WANT_REENT_SMALL
+#ifndef _REENT_SMALL
+#define _REENT_SMALL
+#endif
+#endif
+
+/* If _MB_EXTENDED_CHARSETS_ALL is set, we want all of the extended
+   charsets.  The extended charsets add a few functions and a couple
+   of tables of a few K each. */
+#ifdef _MB_EXTENDED_CHARSETS_ALL
+#define _MB_EXTENDED_CHARSETS_ISO 1
+#define _MB_EXTENDED_CHARSETS_WINDOWS 1
+#endif
+
 #endif /* __SYS_CONFIG_H__ */
