@@ -1,6 +1,6 @@
 /* cygheap.h: Cygwin heap manager.
 
-   Copyright 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
+   Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -8,26 +8,8 @@ This software is a copyrighted work licensed under the terms of the
 Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
-#undef cfree
-
-enum cygheap_types
-{
-  HEAP_FHANDLER,
-  HEAP_STR,
-  HEAP_ARGV,
-  HEAP_BUF,
-  HEAP_MOUNT,
-  HEAP_SIGS,
-  HEAP_ARCHETYPES,
-  HEAP_TLS,
-  HEAP_1_START,
-  HEAP_1_STR,
-  HEAP_1_ARGV,
-  HEAP_1_BUF,
-  HEAP_1_EXEC,
-  HEAP_1_MAX = 100,
-  HEAP_MMAP = 200
-};
+#include "hires.h"
+#include "cygheap_malloc.h"
 
 #define incygheap(s) (cygheap && ((char *) (s) >= (char *) cygheap) && ((char *) (s) <= ((char *) cygheap_max)))
 
@@ -48,6 +30,7 @@ struct cygheap_root_mount_info
   unsigned posix_pathlen;
   char native_path[CYG_MAX_PATH];
   unsigned native_pathlen;
+  bool caseinsensitive;
 };
 
 /* CGF: FIXME This doesn't belong here */
@@ -63,7 +46,8 @@ public:
   {
     if (!m)
       return 1;
-    return path_prefix_p (m->posix_path, path, m->posix_pathlen);
+    return path_prefix_p (m->posix_path, path, m->posix_pathlen,
+			  m->caseinsensitive);
   }
   bool ischroot_native (const char *path)
   {
@@ -82,7 +66,7 @@ public:
     return p;
   }
   bool exists () {return !!m;}
-  void set (const char *, const char *);
+  void set (const char *, const char *, bool);
   size_t posix_length () const { return m->posix_pathlen; }
   const char *posix_path () const { return m->posix_path; }
   size_t native_length () const { return m->native_pathlen; }
@@ -101,7 +85,7 @@ class cygheap_user
   /* Extendend user information.
      The information is derived from the internal_getlogin call
      when on a NT system. */
-  char  *pname;         /* user's name */
+  char  *pname;		/* user's name */
   char  *plogsrv;       /* Logon server, may be FQDN */
   char  *pdomain;       /* Logon domain of the user */
   char  *homedrive;	/* User's home drive */
@@ -122,7 +106,11 @@ public:
      to `set_impersonation_token()'. */
   HANDLE external_token;
   HANDLE internal_token;
-  HANDLE current_token;
+  HANDLE curr_primary_token;
+  HANDLE curr_imp_token;
+  bool ext_token_is_restricted;  /* external_token is restricted token */
+  bool curr_token_is_restricted; /* curr_primary_token is restricted token */
+  bool setuid_to_restricted;     /* switch to restricted token by setuid () */
 
   /* CGF 2002-06-27.  I removed the initializaton from this constructor
      since this class is always allocated statically.  That means that everything
@@ -170,43 +158,46 @@ public:
   PSID saved_sid () { return saved_cygsid; }
   const char *ontherange (homebodies what, struct passwd * = NULL);
 #define NO_IMPERSONATION NULL
-  bool issetuid () const { return current_token != NO_IMPERSONATION; }
-  HANDLE token () { return current_token; }
+  bool issetuid () const { return curr_imp_token != NO_IMPERSONATION; }
+  HANDLE primary_token () { return curr_primary_token; }
+  HANDLE imp_token () { return curr_imp_token; }
   void deimpersonate ()
   {
-    if (issetuid ())
-      RevertToSelf ();
+    RevertToSelf ();
   }
-  void reimpersonate ()
+  bool reimpersonate ()
   {
-    if (issetuid ()
-	&& !ImpersonateLoggedOnUser (token ()))
-      system_printf ("ImpersonateLoggedOnUser: %E");
+    if (issetuid ())
+      return ImpersonateLoggedOnUser (primary_token ());
+    return true;
   }
   bool has_impersonation_tokens ()
     { return external_token != NO_IMPERSONATION
 	     || internal_token != NO_IMPERSONATION
-	     || current_token != NO_IMPERSONATION; }
+	     || curr_primary_token != NO_IMPERSONATION; }
   void close_impersonation_tokens ()
   {
-    if (current_token != NO_IMPERSONATION)
-      {
-	if( current_token != external_token && current_token != internal_token)
-	  CloseHandle (current_token);
-	current_token = NO_IMPERSONATION;
-      }
+    if (curr_imp_token != NO_IMPERSONATION)
+      CloseHandle (curr_imp_token);
+    if (curr_primary_token != NO_IMPERSONATION
+    	&& curr_primary_token != external_token
+	&& curr_primary_token != internal_token)
+      CloseHandle (curr_primary_token);
     if (external_token != NO_IMPERSONATION)
-      {
-	CloseHandle (external_token);
-	external_token = NO_IMPERSONATION;
-      }
+      CloseHandle (external_token);
     if (internal_token != NO_IMPERSONATION)
-      {
-	CloseHandle (internal_token);
-	internal_token = NO_IMPERSONATION;
-      }
+      CloseHandle (internal_token);
   }
-  const char *cygheap_user::test_uid (char *&, const char *, size_t)
+  PWCHAR get_windows_id (PWCHAR buf)
+  {
+    return effec_cygsid.string (buf);
+  }
+  char *get_windows_id (char *buf)
+  {
+    return effec_cygsid.string (buf);
+  }
+
+  const char *test_uid (char *&, const char *, size_t)
     __attribute__ ((regparm (3)));
 };
 
@@ -214,36 +205,45 @@ public:
 
 class muto;
 
-struct cwdstuff
+class cwdstuff
 {
+private:
   char *posix;
-  char *win32;
-  DWORD hash;
+  HANDLE dir;
+public:
+  UNICODE_STRING win32;
   DWORD drive_length;
-  muto *cwd_lock;
-  char *get (char *, int = 1, int = 0, unsigned = CYG_MAX_PATH);
-  DWORD get_hash ();
-  DWORD get_drive (char * dst) 
+  static muto cwd_lock;
+  const char *get_posix () const { return posix; };
+  void reset_posix (wchar_t *);
+  char *get (char *, int = 1, int = 0, unsigned = NT_MAX_PATH);
+  HANDLE get_handle () { return dir; }
+  DWORD get_drive (char * dst)
   {
-    get_initial (); 
-    memcpy (dst, win32, drive_length); 
-    return drive_length;
+    cwd_lock.acquire ();
+    DWORD ret = sys_wcstombs (dst, NT_MAX_PATH, win32.Buffer, drive_length);
+    cwd_lock.release ();
+    return ret;
   }
   void init ();
-  void fixup_after_exec (char *, char *, DWORD);
-  bool get_initial ();
-  int set (const char *, const char *, bool);
+  int set (PUNICODE_STRING, const char *, bool);
 };
 
 #ifdef DEBUGGING
 struct cygheap_debug
 {
   handle_list starth;
-  handle_list *endh;
   handle_list freeh[500];
 };
 #endif
 
+struct cygheap_locale
+{
+  mbtowc_p mbtowc;
+  wctomb_p wctomb;
+  char charset[ENCODING_LEN + 1];
+};
+
 struct user_heap_info
 {
   void *base;
@@ -251,9 +251,22 @@ struct user_heap_info
   void *top;
   void *max;
   unsigned chunk;
+  unsigned slop;
 };
 
-struct init_cygheap
+struct hook_chain
+{
+  void **loc;
+  const void *func;
+  struct hook_chain *next;
+};
+
+struct mini_cygheap
+{
+  cygheap_locale locale;
+};
+
+struct init_cygheap: public mini_cygheap
 {
   _cmalloc_entry *chain;
   char *buckets[32];
@@ -261,13 +274,9 @@ struct init_cygheap
   cygheap_user user;
   user_heap_info user_heap;
   mode_t umask;
-  HANDLE shared_h;
   HANDLE console_h;
-  HANDLE mt_h;
-  char *cygwin_regname;
   cwdstuff cwd;
   dtable fdtab;
-  const char *shared_prefix;
 #ifdef DEBUGGING
   cygheap_debug debug;
 #endif
@@ -279,11 +288,19 @@ struct init_cygheap
 #endif
   struct _cygtls **threadlist;
   size_t sthreads;
-  int open_fhs;
   pid_t pid;			/* my pid */
+  HANDLE pid_handle;		/* handle for my pid */
+  struct {			/* Equivalent to using LIST_HEAD. */
+    struct inode_t *lh_first;
+  } inode_list;			/* Global inode pointer for adv. locking. */
+  hook_chain hooks;
   void close_ctty ();
+  int manage_console_count (const char *, int, bool = false) __attribute__ ((regparm (3)));
+private:
+  int console_count;
 };
 
+
 #define _CYGHEAPSIZE_SLOP (128 * 1024)
 #define CYGHEAPSIZE (sizeof (init_cygheap) + (20000 * sizeof (fhandler_union)) + _CYGHEAPSIZE_SLOP)
 #define CYGHEAPSIZE_MIN (sizeof (init_cygheap) + (10000 * sizeof (fhandler_union)))
@@ -311,6 +328,7 @@ class cygheap_fdmanip
   operator int &() {return fd;}
   operator fhandler_base* &() {return *fh;}
   operator fhandler_socket* () const {return reinterpret_cast<fhandler_socket *> (*fh);}
+  operator fhandler_pipe* () const {return reinterpret_cast<fhandler_pipe *> (*fh);}
   void operator = (fhandler_base *fh) {*this->fh = fh;}
   fhandler_base *operator -> () const {return *fh;}
   bool isopen () const
@@ -374,18 +392,29 @@ class cygheap_fdget : public cygheap_fdmanip
   }
 };
 
-class child_info;
-void *__stdcall cygheap_setup_for_child (child_info *ci, bool dup_later) __attribute__ ((regparm(2)));
-void __stdcall cygheap_setup_for_child_cleanup (void *, child_info *, bool) __attribute__ ((regparm(3)));
+class cygheap_fdenum : public cygheap_fdmanip
+{
+ public:
+  cygheap_fdenum (bool lockit = false)
+  {
+    locked = lockit;
+    if (lockit)
+      cygheap->fdtab.lock ();
+    fd = -1;
+  }
+  int next ()
+  {
+    while (++fd < (int) cygheap->fdtab.size)
+      if (*(fh = cygheap->fdtab + fd) != NULL)
+	return fd;
+    return -1;
+  }
+  void rewind ()
+  {
+    fd = -1;
+  }
+};
+
 void __stdcall cygheap_fixup_in_child (bool);
-extern "C" {
-void __stdcall cfree (void *) __attribute__ ((regparm(1)));
-void *__stdcall cmalloc (cygheap_types, DWORD) __attribute__ ((regparm(2)));
-void *__stdcall crealloc (void *, DWORD) __attribute__ ((regparm(2)));
-void *__stdcall ccalloc (cygheap_types, DWORD, DWORD) __attribute__ ((regparm(3)));
-char *__stdcall cstrdup (const char *) __attribute__ ((regparm(1)));
-char *__stdcall cstrdup1 (const char *) __attribute__ ((regparm(1)));
-void __stdcall cfree_and_set (char *&, char * = NULL) __attribute__ ((regparm(2)));
 void __stdcall cygheap_init ();
-extern DWORD _cygheap_start;
-}
+extern char _cygheap_start[] __attribute__((section(".idata")));
