@@ -1,6 +1,6 @@
 /* cygmalloc.h: cygwin DLL malloc stuff
 
-   Copyright 2002, 2003, 2004 Red Hat, Inc.
+   Copyright 2002, 2003, 2004, 2005, 2007 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -23,11 +23,13 @@ int dlmallopt (int p, int v) __attribute__ ((regparm (2)));
 void dlmalloc_stats ();
 
 #ifndef __INSIDE_CYGWIN__
+extern "C" void __set_ENOMEM ();
+# define MALLOC_FAILURE_ACTION	__set_ENOMEM ()
 # define USE_DL_PREFIX 1
 #else
-# define __malloc_lock() mallock->acquire ()
-# define __malloc_unlock() mallock->release ()
-extern muto *mallock;
+# define __malloc_lock() mallock.acquire ()
+# define __malloc_unlock() mallock.release ()
+extern muto mallock;
 #endif
 #ifdef __cplusplus
 }
