@@ -1,6 +1,7 @@
 /* dll_init.h
 
-   Copyright 1998, 1999, 2000, 2001 Red Hat, Inc.
+   Copyright 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2007, 2008,
+   2009 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -49,11 +50,19 @@ struct dll
   per_module p;
   HMODULE handle;
   int count;
+  bool has_dtors;
   dll_type type;
-  int namelen;
-  char name[CYG_MAX_PATH + 1];
+  WCHAR name[1];
   void detach ();
   int init ();
+  void run_dtors ()
+  {
+    if (has_dtors)
+      {
+	has_dtors = 0;
+	p.run_dtors ();
+      }
+  }
 };
 
 #define MAX_DLL_BEFORE_INIT     100
@@ -68,11 +77,12 @@ public:
   int tot;
   int loaded_dlls;
   int reload_on_fork;
-  dll *operator [] (const char *name);
+  dll *operator [] (const PWCHAR name);
   dll *alloc (HINSTANCE, per_process *, dll_type);
+  dll *find (void *);
   void detach (void *);
   void init ();
-  void load_after_fork (HANDLE, dll *);
+  void load_after_fork (HANDLE);
   dll *inext ()
   {
     while ((hold = hold->next))
@@ -86,6 +96,13 @@ public:
     hold = &start;
     return inext ();
   }
+  friend void dll_global_dtors ();
 };
 
 extern dll_list dlls;
+void dll_global_dtors ();
+
+/* These probably belong in a newlib header but we can keep them here
+   for now.  */
+extern "C" int __cxa_atexit(void (*)(void*), void*, void*);
+extern "C" int __cxa_finalize(void*);
