@@ -1,6 +1,7 @@
 /* errno.cc: errno-related functions
 
-   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+   2006, 2008, 2009 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -12,9 +13,6 @@ details. */
 #define sys_nerr FOOsys_nerr
 #define _sys_errlist FOO_sys_errlist
 #include "winsup.h"
-#include <stdio.h>
-#include "cygerrno.h"
-#include "thread.h"
 #include "cygtls.h"
 #undef _sys_nerr
 #undef sys_nerr
@@ -34,98 +32,119 @@ static NO_COPY struct
 } errmap[] =
 {
   /* FIXME: Some of these choices are arbitrary! */
-  X (INVALID_FUNCTION,		EBADRQC),
-  X (FILE_NOT_FOUND,		ENOENT),
-  X (PATH_NOT_FOUND,		ENOENT),
-  X (TOO_MANY_OPEN_FILES,	EMFILE),
   X (ACCESS_DENIED,		EACCES),
-  X (INVALID_HANDLE,		EBADF),
-  X (NOT_ENOUGH_MEMORY,		ENOMEM),
-  X (INVALID_DATA,		EINVAL),
-  X (OUTOFMEMORY,		ENOMEM),
-  X (INVALID_DRIVE,		ENODEV),
-  X (NOT_SAME_DEVICE,		EXDEV),
-  X (NO_MORE_FILES,		ENMFILE),
-  X (WRITE_PROTECT,		EROFS),
-  X (BAD_UNIT,			ENODEV),
-  X (SHARING_VIOLATION,		EBUSY),
-  X (LOCK_VIOLATION,		EACCES),
-  X (SHARING_BUFFER_EXCEEDED,	ENOLCK),
-  X (HANDLE_EOF,		ENODATA),
-  X (HANDLE_DISK_FULL,		ENOSPC),
-  X (NOT_SUPPORTED,		ENOSYS),
-  X (REM_NOT_LIST,		ENONET),
-  X (DUP_NAME,			ENOTUNIQ),
-  X (BAD_NETPATH,		ENOSHARE),
-  X (BAD_NET_NAME,		ENOSHARE),
-  X (FILE_EXISTS,		EEXIST),
-  X (CANNOT_MAKE,		EPERM),
-  X (INVALID_PARAMETER,		EINVAL),
-  X (NO_PROC_SLOTS,		EAGAIN),
-  X (BROKEN_PIPE,		EPIPE),
-  X (OPEN_FAILED,		EIO),
-  X (NO_MORE_SEARCH_HANDLES,	ENFILE),
-  X (CALL_NOT_IMPLEMENTED,	ENOSYS),
-  X (INVALID_NAME,		ENOENT),
-  X (WAIT_NO_CHILDREN,		ECHILD),
-  X (CHILD_NOT_COMPLETE,	EBUSY),
-  X (DIR_NOT_EMPTY,		ENOTEMPTY),
-  X (SIGNAL_REFUSED,		EIO),
-  X (BAD_PATHNAME,		ENOENT),
-  X (SIGNAL_PENDING,		EBUSY),
-  X (MAX_THRDS_REACHED,		EAGAIN),
-  X (BUSY,			EBUSY),
-  X (ALREADY_EXISTS,		EEXIST),
-  X (NO_SIGNAL_SENT,		EIO),
-  X (FILENAME_EXCED_RANGE,	EINVAL),
-  X (META_EXPANSION_TOO_LONG,	EINVAL),
-  X (INVALID_SIGNAL_NUMBER,	EINVAL),
-  X (THREAD_1_INACTIVE,		EINVAL),
-  X (BAD_PIPE,			EINVAL),
-  X (PIPE_BUSY,			EBUSY),
-  X (NO_DATA,			EPIPE),
-  X (PIPE_NOT_CONNECTED,	ECOMM),
-  X (MORE_DATA,			EAGAIN),
-  X (DIRECTORY,			ENOTDIR),
-  X (PIPE_CONNECTED,		EBUSY),
-  X (PIPE_LISTENING,		ECOMM),
-  X (NO_TOKEN,			EINVAL),
-  X (PROCESS_ABORTED,		EFAULT),
-  X (BAD_DEVICE,		ENODEV),
-  X (BAD_USERNAME,		EINVAL),
-  X (NOT_CONNECTED,		ENOLINK),
-  X (OPEN_FILES,		EAGAIN),
   X (ACTIVE_CONNECTIONS,	EAGAIN),
+  X (ALREADY_EXISTS,		EEXIST),
+  X (BAD_DEVICE,		ENODEV),
+  X (BAD_NETPATH,		ENOENT),
+  X (BAD_NET_NAME,		ENOENT),
+  X (BAD_PATHNAME,		ENOENT),
+  X (BAD_PIPE,			EINVAL),
+  X (BAD_UNIT,			ENODEV),
+  X (BAD_USERNAME,		EINVAL),
+  X (BEGINNING_OF_MEDIA,	EIO),
+  X (BROKEN_PIPE,		EPIPE),
+  X (BUSY,			EBUSY),
+  X (BUS_RESET,			EIO),
+  X (CALL_NOT_IMPLEMENTED,	ENOSYS),
+  X (CANNOT_MAKE,		EPERM),
+  X (CHILD_NOT_COMPLETE,	EBUSY),
+  X (COMMITMENT_LIMIT,		EAGAIN),
+  X (CRC,			EIO),
+  X (DEVICE_DOOR_OPEN,		EIO),
   X (DEVICE_IN_USE,		EAGAIN),
-  X (INVALID_AT_INTERRUPT_TIME,	EINTR),
-  X (IO_DEVICE,			EIO),
-  X (NOT_OWNER,			EPERM),
+  X (DEVICE_REQUIRES_CLEANING,	EIO),
+  X (DIRECTORY,			ENOTDIR),
+  X (DIR_NOT_EMPTY,		ENOTEMPTY),
+  X (DISK_CORRUPT,		EIO),
+  X (DISK_FULL,			ENOSPC),
+  X (DUP_NAME,			ENOTUNIQ),
+  X (EAS_DIDNT_FIT,		ENOSPC),
+  X (EAS_NOT_SUPPORTED,		ENOTSUP),
+  X (EA_LIST_INCONSISTENT,	EINVAL),
+  X (EA_TABLE_FULL,		ENOSPC),
   X (END_OF_MEDIA,		ENOSPC),
   X (EOM_OVERFLOW,		EIO),
-  X (BEGINNING_OF_MEDIA,	EIO),
-  X (SETMARK_DETECTED,		EIO),
-  X (NO_DATA_DETECTED,		EIO),
-  X (POSSIBLE_DEADLOCK,		EDEADLOCK),
-  X (CRC,			EIO),
-  X (NEGATIVE_SEEK,		EINVAL),
-  X (NOT_READY,			ENOMEDIUM),
-  X (DISK_FULL,			ENOSPC),
-  X (NOACCESS,			EFAULT),
-  X (FILE_INVALID,		ENXIO),
-  X (INVALID_ADDRESS,		EOVERFLOW),
-  X (INVALID_BLOCK_LENGTH,	EIO),
-  X (BUS_RESET,			EIO),
   X (FILEMARK_DETECTED,		EIO),
-  X (NO_MEDIA_IN_DRIVE,		ENOMEDIUM),
-  X (DEVICE_REQUIRES_CLEANING,	EIO),
-  X (DEVICE_DOOR_OPEN,		EIO),
+  X (FILENAME_EXCED_RANGE,	ENAMETOOLONG),
+  X (FILE_CORRUPT,		EEXIST),
+  X (FILE_EXISTS,		EEXIST),
+  X (FILE_INVALID,		ENXIO),
+  X (FILE_NOT_FOUND,		ENOENT),
+  X (HANDLE_DISK_FULL,		ENOSPC),
+  X (HANDLE_EOF,		ENODATA),
+  X (INVALID_ADDRESS,		EINVAL),
+  X (INVALID_AT_INTERRUPT_TIME,	EINTR),
+  X (INVALID_BLOCK_LENGTH,	EIO),
+  X (INVALID_DATA,		EINVAL),
+  X (INVALID_DRIVE,		ENODEV),
+  X (INVALID_EA_NAME,		EINVAL),
+  X (INVALID_FUNCTION,		EBADRQC),
+  X (INVALID_HANDLE,		EBADF),
+  X (INVALID_NAME,		ENOENT),
+  X (INVALID_PARAMETER,		EINVAL),
+  X (INVALID_SIGNAL_NUMBER,	EINVAL),
+  X (IO_DEVICE,			EIO),
   X (IO_PENDING,		EAGAIN),
+  X (LOCK_VIOLATION,		EACCES),
+  X (MAX_THRDS_REACHED,		EAGAIN),
+  X (META_EXPANSION_TOO_LONG,	EINVAL),
+  X (MOD_NOT_FOUND,		ENOENT),
+  X (MORE_DATA,			EMSGSIZE),
+  X (NEGATIVE_SEEK,		EINVAL),
+  X (NETNAME_DELETED,		ENOENT),
+  X (NOACCESS,			EFAULT),
+  X (NONPAGED_SYSTEM_RESOURCES,	EAGAIN),
+  X (NONE_MAPPED,		EINVAL),
+  X (NOT_CONNECTED,		ENOLINK),
+  X (NOT_ENOUGH_MEMORY,		ENOMEM),
+  X (NOT_OWNER,			EPERM),
+  X (NOT_READY,			ENOMEDIUM),
+  X (NOT_SAME_DEVICE,		EXDEV),
+  X (NOT_SUPPORTED,		ENOSYS),
+  X (NO_DATA,			EPIPE),
+  X (NO_DATA_DETECTED,		EIO),
+  X (NO_MEDIA_IN_DRIVE,		ENOMEDIUM),
+  X (NO_MORE_FILES,		ENMFILE),
+  X (NO_MORE_ITEMS,		ENMFILE),
+  X (NO_MORE_SEARCH_HANDLES,	ENFILE),
+  X (NO_PROC_SLOTS,		EAGAIN),
+  X (NO_SIGNAL_SENT,		EIO),
+  X (NO_SYSTEM_RESOURCES,	EAGAIN),
+  X (NO_TOKEN,			EINVAL),
+  X (OPEN_FAILED,		EIO),
+  X (OPEN_FILES,		EAGAIN),
+  X (OUTOFMEMORY,		ENOMEM),
+  X (PAGED_SYSTEM_RESOURCES,	EAGAIN),
+  X (PAGEFILE_QUOTA,		EAGAIN),
+  X (PATH_NOT_FOUND,		ENOENT),
+  X (PIPE_BUSY,			EBUSY),
+  X (PIPE_CONNECTED,		EBUSY),
+  X (PIPE_LISTENING,		ECOMM),
+  X (PIPE_NOT_CONNECTED,	ECOMM),
+  X (POSSIBLE_DEADLOCK,		EDEADLOCK),
+  X (PROCESS_ABORTED,		EFAULT),
+  X (PROC_NOT_FOUND,		ESRCH),
+  X (REM_NOT_LIST,		ENONET),
+  X (SETMARK_DETECTED,		EIO),
+  X (SHARING_BUFFER_EXCEEDED,	ENOLCK),
+  X (SHARING_VIOLATION,		EBUSY),
+  X (SIGNAL_PENDING,		EBUSY),
+  X (SIGNAL_REFUSED,		EIO),
+  X (THREAD_1_INACTIVE,		EINVAL),
   X (TOO_MANY_LINKS,		EMLINK),
+  X (TOO_MANY_OPEN_FILES,	EMFILE),
+  X (WAIT_NO_CHILDREN,		ECHILD),
+  X (WORKING_SET_QUOTA,		EAGAIN),
+  X (WRITE_PROTECT,		EROFS),
+  X (SEEK,			EINVAL),
+  X (SECTOR_NOT_FOUND,		EINVAL),
+  X (IO_INCOMPLETE,		EAGAIN),
   { 0, NULL, 0}
 };
 
 extern "C" {
-const char __declspec(dllexport) * _sys_errlist[] NO_COPY_INIT =
+const char *_sys_errlist[] NO_COPY_INIT =
 {
 /* NOERROR 0 */		  "No error",
 /* EPERM 1 */		  "Operation not permitted",
@@ -174,9 +193,9 @@ const char __declspec(dllexport) * _sys_errlist[] NO_COPY_INIT =
 /* EL2HLT 44 */		  "Level 2 halted",
 /* EDEADLK 45 */	  "Resource deadlock avoided",
 /* ENOLCK 46 */		  "No locks available",
-		  	  "error 47",
-		  	  "error 48",
-		  	  "error 49",
+			  "error 47",
+			  "error 48",
+			  "error 49",
 /* EBADE 50 */		  "Invalid exchange",
 /* EBADR 51 */		  "Invalid request descriptor",
 /* EXFULL 52 */		  "Exchange full",
@@ -185,8 +204,8 @@ const char __declspec(dllexport) * _sys_errlist[] NO_COPY_INIT =
 /* EBADSLT 55 */	  "Invalid slot",
 /* EDEADLOCK 56 */	  "File locking deadlock error",
 /* EBFONT 57 */		  "Bad font file format",
-		  	  "error 58",
-		  	  "error 59",
+			  "error 58",
+			  "error 59",
 /* ENOSTR 60 */		  "Device not a stream",
 /* ENODATA 61 */	  "No data available",
 /* ETIME 62 */		  "Timer expired",
@@ -199,13 +218,13 @@ const char __declspec(dllexport) * _sys_errlist[] NO_COPY_INIT =
 /* ESRMNT 69 */		  "Srmount error",
 /* ECOMM 70 */		  "Communication error on send",
 /* EPROTO 71 */		  "Protocol error",
-		  	  "error 72",
-		  	  "error 73",
+			  "error 72",
+			  "error 73",
 /* EMULTIHOP 74 */	  "Multihop attempted",
 /* ELBIN 75 */		  "Inode is remote (not really error)",
 /* EDOTDOT 76 */	  "RFS specific error",
 /* EBADMSG 77 */	  "Bad message",
-		  	  "error 78",
+			  "error 78",
 /* EFTYPE 79 */		  "Inappropriate file type or format",
 /* ENOTUNIQ 80 */	  "Name not unique on network",
 /* EBADFD 81 */		  "File descriptor in bad state",
@@ -220,22 +239,22 @@ const char __declspec(dllexport) * _sys_errlist[] NO_COPY_INIT =
 /* ENOTEMPTY 90	*/	  "Directory not empty",
 /* ENAMETOOLONG 91 */	  "File name too long",
 /* ELOOP 92 */		  "Too many levels of symbolic links",
-		  	  "error 93",
-		  	  "error 94",
+			  "error 93",
+			  "error 94",
 /* EOPNOTSUPP 95 */	  "Operation not supported",
 /* EPFNOSUPPORT 96 */	  "Protocol family not supported",
-		  	  "error 97",
-		  	  "error 98",
-		  	  "error 99",
-		  	  "error 100",
-		  	  "error 101",
-		  	  "error 102",
-		  	  "error 103",
+			  "error 97",
+			  "error 98",
+			  "error 99",
+			  "error 100",
+			  "error 101",
+			  "error 102",
+			  "error 103",
 /* ECONNRESET 104 */	  "Connection reset by peer",
 /* ENOBUFS 105 */	  "No buffer space available",
 /* EAFNOSUPPORT 106 */	  "Address family not supported by protocol",
 /* EPROTOTYPE 107 */	  "Protocol wrong type for socket",
-/* ENOTSOCK 108 */ 	  "Socket operation on non-socket",
+/* ENOTSOCK 108 */	  "Socket operation on non-socket",
 /* ENOPROTOOPT 109 */	  "Protocol not available",
 /* ESHUTDOWN 110 */	  "Cannot send after transport endpoint shutdown",
 /* ECONNREFUSED 111 */	  "Connection refused",
@@ -261,12 +280,16 @@ const char __declspec(dllexport) * _sys_errlist[] NO_COPY_INIT =
 /* EUSERS 131 */	  "Too many users",
 /* EDQUOT 132 */	  "Disk quota exceeded",
 /* ESTALE 133 */	  "Stale NFS file handle",
-/* ENOTSUP 134 */  	  "Not supported",
+/* ENOTSUP 134 */	  "Not supported",
 /* ENOMEDIUM 135 */	  "No medium found",
-/* ENOSHARE 136 */  	  "No such host or network path",
+/* ENOSHARE 136 */	  "No such host or network path",
 /* ECASECLASH 137 */	  "Filename exists with different case",
 /* EILSEQ 138 */	  "Invalid or incomplete multibyte or wide character",
-/* EOVERFLOW 139 */	  "Value too large for defined data type"
+/* EOVERFLOW 139 */	  "Value too large for defined data type",
+/* ECANCELED 140 */	  "Operation canceled",
+/* ENOTRECOVERABLE 141 */ "State not recoverable",
+/* EOWNERDEAD 142 */	  "Previous owner died",
+/* ESTRPIPE 143 */	  "Streams pipe error"
 };
 
 int NO_COPY_INIT _sys_nerr = sizeof (_sys_errlist) / sizeof (_sys_errlist[0]);
@@ -294,7 +317,6 @@ seterrno_from_win_error (const char *file, int line, DWORD code)
 {
   syscall_printf ("%s:%d windows error %d", file, line, code);
   set_errno (geterrno_from_win_error (code, EACCES));
-  return;
 }
 
 /* seterrno: Set `errno' based on GetLastError (). */
@@ -306,27 +328,38 @@ seterrno (const char *file, int line)
 
 extern char *_user_strerror _PARAMS ((int));
 
-/* FIXME: Why is strerror() a long switch and not just:
-	return sys_errlist[errnum];
-	(or moral equivalent).
-	Some entries in sys_errlist[] don't match the corresponding
-	entries in strerror().  This seems odd.
-*/
+static char *
+strerror_worker (int errnum)
+{
+  char *res;
+  if (errnum >= 0 && errnum < _sys_nerr)
+    res = (char *) _sys_errlist [errnum];
+  else
+    res = NULL;
+  return res;
+}
 
-/* CYGWIN internal */
 /* strerror: convert from errno values to error strings */
 extern "C" char *
 strerror (int errnum)
 {
-  const char *error;
-  if (errnum < _sys_nerr)
-    error = _sys_errlist [errnum];
-  else
-    {
-      __small_sprintf (_my_tls.locals.strerror_buf, "error %d", errnum);
-      error = _my_tls.locals.strerror_buf;
-    }
-  /* FIXME: strerror should really be const in the appropriate newlib
-     include files. */
-  return (char *) error;
+  char *errstr = strerror_worker (errnum);
+  if (!errstr)
+    __small_sprintf (errstr = _my_tls.locals.strerror_buf, "Unknown error %u",
+		     (unsigned) errnum);
+  return errstr;
 }
+
+#if 0
+extern "C" int
+strerror_r (int errnum, char *buf, size_t n)
+{
+  char *errstr = strerror_worker (errnum);
+  if (!errstr)
+    return EINVAL;
+  if (strlen (errstr) >= n)
+    return ERANGE;
+  strcpy (buf, errstr);
+  return 0;
+}
+#endif
