@@ -1,6 +1,6 @@
 /* Fhandler_dev_dsp: code to emulate OSS sound model /dev/dsp
 
-   Copyright 2001, 2002, 2003, 2004 Red Hat, Inc
+   Copyright 2001, 2002, 2003, 2004, 2008 Red Hat, Inc
 
    Written by Andy Younger (andy@snoogie.demon.co.uk)
    Extended by Gerd Spalink (Gerd.Spalink@t-online.de)
@@ -13,10 +13,7 @@ Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
 #include "winsup.h"
-#include <stdio.h>
-#include <windows.h>
 #include <sys/soundcard.h>
-#include <mmsystem.h>
 #include "cygerrno.h"
 #include "security.h"
 #include "path.h"
@@ -945,7 +942,7 @@ fhandler_dev_dsp::fhandler_dev_dsp ():
 int
 fhandler_dev_dsp::open (int flags, mode_t mode)
 {
-  if (cygheap->fdtab.find_archetype (pc.dev))
+  if (cygheap->fdtab.find_archetype (dev ()))
     {
       set_errno (EBUSY);
       return 0;
@@ -983,7 +980,7 @@ fhandler_dev_dsp::open (int flags, mode_t mode)
       nohandle (true);
 
       // FIXME: Do this better someday
-      fhandler_dev_dsp *arch = (fhandler_dev_dsp *) cmalloc (HEAP_ARCHETYPES, sizeof (*this));
+      fhandler_dev_dsp *arch = (fhandler_dev_dsp *) cmalloc_abort (HEAP_ARCHETYPES, sizeof (*this));
       archetype = arch;
       *((fhandler_dev_dsp **) cygheap->fdtab.add_archetype ()) = arch;
       *arch = *this;
@@ -1000,7 +997,7 @@ fhandler_dev_dsp::open (int flags, mode_t mode)
 #define IS_WRITE() ((get_flags() & O_ACCMODE) != O_RDONLY)
 #define IS_READ() ((get_flags() & O_ACCMODE) != O_WRONLY)
 
-int
+ssize_t __stdcall
 fhandler_dev_dsp::write (const void *ptr, size_t len)
 {
   debug_printf ("ptr=%08x len=%d", ptr, len);
@@ -1010,24 +1007,25 @@ fhandler_dev_dsp::write (const void *ptr, size_t len)
   int len_s = len;
   const char *ptr_s = static_cast <const char *> (ptr);
 
-  if (!audio_out_)
-    if (IS_WRITE ())
-      {
-	debug_printf ("Allocating");
-	if (!(audio_out_ = new Audio_out))
-	  return -1;
-
-	/* check for wave file & get parameters & skip header if possible. */
-
-	if (audio_out_->parsewav (ptr_s, len_s,
-				  audiofreq_, audiobits_, audiochannels_))
-	  debug_printf ("=> ptr_s=%08x len_s=%d", ptr_s, len_s);
-      }
-    else
-      {
-	set_errno (EBADF); // device was opened for read?
+  if (audio_out_)
+    /* nothing to do */;
+  else if (IS_WRITE ())
+    {
+      debug_printf ("Allocating");
+      if (!(audio_out_ = new Audio_out))
 	return -1;
-      }
+
+      /* check for wave file & get parameters & skip header if possible. */
+
+      if (audio_out_->parsewav (ptr_s, len_s,
+				audiofreq_, audiobits_, audiochannels_))
+	debug_printf ("=> ptr_s=%08x len_s=%d", ptr_s, len_s);
+    }
+  else
+    {
+      set_errno (EBADF); // device was opened for read?
+      return -1;
+    }
 
   /* Open audio device properly with callbacks.
      Private parameters were set in call to parsewav.
@@ -1049,23 +1047,24 @@ fhandler_dev_dsp::read (void *ptr, size_t& len)
   if ((fhandler_dev_dsp *) archetype != this)
     return ((fhandler_dev_dsp *)archetype)->read(ptr, len);
 
-  if (!audio_in_)
-    if (IS_READ ())
-      {
-	debug_printf ("Allocating");
-	if (!(audio_in_ = new Audio_in))
-	  {
-	    len = (size_t)-1;
-	    return;
-	  }
-	audio_in_->setconvert (audioformat_);
-      }
-    else
-      {
-	len = (size_t)-1;
-	set_errno (EBADF); // device was opened for write?
-	return;
-      }
+  if (audio_in_)
+    /* nothing to do */;
+  else if (IS_READ ())
+    {
+      debug_printf ("Allocating");
+      if (!(audio_in_ = new Audio_in))
+	{
+	  len = (size_t)-1;
+	  return;
+	}
+      audio_in_->setconvert (audioformat_);
+    }
+  else
+    {
+      len = (size_t)-1;
+      set_errno (EBADF); // device was opened for write?
+      return;
+    }
 
   /* Open audio device properly with callbacks.
      This is a noop when there are successive reads in the same process */
@@ -1077,7 +1076,6 @@ fhandler_dev_dsp::read (void *ptr, size_t& len)
     }
 
   audio_in_->read ((char *)ptr, (int&)len);
-  return;
 }
 
 _off64_t
@@ -1109,17 +1107,20 @@ fhandler_dev_dsp::close_audio_out (bool immediately)
 }
 
 int
-fhandler_dev_dsp::close (void)
+fhandler_dev_dsp::close ()
 {
   debug_printf ("audio_in=%08x audio_out=%08x",
 		(int)audio_in_, (int)audio_out_);
-  if ((fhandler_dev_dsp *) archetype != this)
-    return ((fhandler_dev_dsp *)archetype)->close ();
-
-  if (--usecount == 0)
+  if (!hExeced)
     {
-      close_audio_in ();
-      close_audio_out (exit_state != ES_NOT_EXITING);
+      if ((fhandler_dev_dsp *) archetype != this)
+	return ((fhandler_dev_dsp *) archetype)->close ();
+
+      if (--usecount == 0)
+	{
+	  close_audio_in ();
+	  close_audio_out (exit_state != ES_NOT_EXITING);
+	}
     }
   return 0;
 }
@@ -1129,6 +1130,7 @@ fhandler_dev_dsp::dup (fhandler_base * child)
 {
   debug_printf ("");
   child->archetype = archetype;
+  child->set_flags (get_flags ());
   archetype->usecount++;
   return 0;
 }
@@ -1342,12 +1344,6 @@ fhandler_dev_dsp::ioctl (unsigned int cmd, void *ptr)
   return -1;
 }
 
-void
-fhandler_dev_dsp::dump ()
-{
-  paranoid_printf ("here");
-}
-
 void
 fhandler_dev_dsp::fixup_after_fork (HANDLE parent)
 { // called from new child process
@@ -1365,11 +1361,14 @@ fhandler_dev_dsp::fixup_after_fork (HANDLE parent)
 void
 fhandler_dev_dsp::fixup_after_exec ()
 {
-  debug_printf ("audio_in=%08x audio_out=%08x",
-		(int)audio_in_, (int)audio_out_);
-  if (archetype != this)
-    return ((fhandler_dev_dsp *)archetype)->fixup_after_exec ();
+  debug_printf ("audio_in=%08x audio_out=%08x, close_on_exec %d",
+		(int) audio_in_, (int) audio_out_, close_on_exec ());
+  if (!close_on_exec ())
+    {
+      if (archetype != this)
+	return ((fhandler_dev_dsp *) archetype)->fixup_after_exec ();
 
-  audio_in_ = NULL;
-  audio_out_ = NULL;
+      audio_in_ = NULL;
+      audio_out_ = NULL;
+    }
 }
