@@ -1,6 +1,7 @@
 /* fhandler_termios.cc
 
-   Copyright 1999, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
+   Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2008, 2009
+   Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -9,18 +10,15 @@ Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
 #include "winsup.h"
-#include <sys/termios.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <ctype.h>
 #include "cygerrno.h"
-#include "security.h"
 #include "path.h"
 #include "fhandler.h"
 #include "sigproc.h"
 #include "pinfo.h"
 #include "tty.h"
-#include "sys/cygwin.h"
+#include "cygtls.h"
 
 /* Common functions shared by tty/console */
 
@@ -31,7 +29,7 @@ fhandler_termios::tcinit (tty_min *this_tc, bool force)
 
   tc = this_tc;
 
-  if (force || !TTYISSETF (INITIALIZED))
+  if (force || !tc->initialized ())
     {
       tc->ti.c_iflag = BRKINT | ICRNL | IXON;
       tc->ti.c_oflag = OPOST | ONLCR;
@@ -58,7 +56,7 @@ fhandler_termios::tcinit (tty_min *this_tc, bool force)
 
       tc->ti.c_ispeed = tc->ti.c_ospeed = B38400;
       tc->pgid = myself->pgid;
-      TTYSETF (INITIALIZED);
+      tc->initialized (true);
     }
 }
 
@@ -72,14 +70,45 @@ fhandler_termios::tcsetpgrp (const pid_t pgid)
       set_errno (EPERM);
       return -1;
     }
-  tc->setpgid (pgid);
-  return 0;
+  int res;
+  while (1)
+    {
+      res = bg_check (-SIGTTOU);
+
+      switch (res)
+	{
+	case bg_ok:
+	  tc->setpgid (pgid);
+	  init_console_handler (tc->gethwnd ());
+	  res = 0;
+	  break;
+	case bg_signalled:
+	  if (_my_tls.call_signal_handler ())
+	    continue;
+	  set_errno (EINTR);
+	  /* fall through intentionally */
+	default:
+	  res = -1;
+	  break;
+	}
+      break;
+    }
+  return res;
 }
 
 int
 fhandler_termios::tcgetpgrp ()
 {
-  return tc->pgid;
+  if (myself->ctty != -1 && myself->ctty == tc->ntty)
+    return tc->pgid;
+  set_errno (ENOTTY);
+  return -1;
+}
+
+int
+fhandler_pty_master::tcgetpgrp ()
+{
+  return myself->ctty != -1 && myself->ctty == tc->ntty ? tc->pgid : 0;
 }
 
 void
@@ -87,19 +116,18 @@ tty_min::kill_pgrp (int sig)
 {
   int killself = 0;
   winpids pids ((DWORD) PID_MAP_RW);
-  siginfo_t si;
+  siginfo_t si = {0};
   si.si_signo = sig;
   si.si_code = SI_KERNEL;
-  si.si_pid = si.si_uid = si.si_errno = 0;
   for (unsigned i = 0; i < pids.npids; i++)
     {
       _pinfo *p = pids[i];
-      if (!proc_exists (p) || p->ctty != ntty || p->pgid != pgid)
+      if (!p->exists () || p->ctty != ntty || p->pgid != pgid)
 	continue;
       if (p == myself)
 	killself++;
       else
-	(void) sig_send (p, si);
+	sig_send (p, si);
     }
   if (killself)
     sig_send (myself, si);
@@ -116,8 +144,8 @@ fhandler_termios::bg_check (int sig)
   if (sig < 0)
     sig = -sig;
 
-  termios_printf ("bg I/O pgid %d, tpgid %d, ctty %d",
-		    myself->pgid, tc->getpgid (), myself->ctty);
+  termios_printf ("bg I/O pgid %d, tpgid %d, %s, ntty tty%d", myself->pgid, tc->getpgid (),
+		  myctty (), tc->ntty);
 
   if (tc->getsid () == 0)
     {
@@ -134,7 +162,7 @@ fhandler_termios::bg_check (int sig)
   int pgid_gone = !pid_exists (myself->pgid);
   int sigs_ignored =
     ((void *) global_sigs[sig].sa_handler == (void *) SIG_IGN) ||
-    (myself->getsigmask () & SIGTOMASK (sig));
+    (_main_tls->sigmask & SIGTOMASK (sig));
 
   if (pgid_gone)
     goto setEIO;
@@ -149,10 +177,9 @@ fhandler_termios::bg_check (int sig)
      by another signal. */
   if (WaitForSingleObject (signal_arrived, 0) != WAIT_OBJECT_0)
     {
-      siginfo_t si;
+      siginfo_t si = {0};
       si.si_signo = sig;
       si.si_code = SI_KERNEL;
-      si.si_pid = si.si_uid = si.si_errno = 0;
       kill_pgrp (myself->pgid, si);
     }
   return bg_signalled;
@@ -293,7 +320,7 @@ fhandler_termios::line_edit (const char *rptr, int nread, termios& ti)
       else if (CCEQ (ti.c_cc[VEOF], c))
 	{
 	  termios_printf ("EOF");
-	  (void) accept_input ();
+	  accept_input ();
 	  ret = line_edit_input_done;
 	  continue;
 	}
@@ -334,13 +361,6 @@ fhandler_termios::line_edit (const char *rptr, int nread, termios& ti)
   return ret;
 }
 
-void
-fhandler_termios::fixup_after_fork (HANDLE parent)
-{
-  fhandler_base::fixup_after_fork (parent);
-  fork_fixup (parent, get_output_handle (), "output_handle");
-}
-
 _off64_t
 fhandler_termios::lseek (_off64_t, int)
 {
