@@ -1,6 +1,7 @@
 /* fhandler_tty.cc
 
-   Copyright 1997, 1998, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
+   Copyright 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+   2006, 2007, 2008, 2009, 2010 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -9,13 +10,8 @@ Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
 #include "winsup.h"
-#include <wingdi.h>
-#include <winuser.h>
-#include <stdio.h>
-#include <unistd.h>
 #include <stdlib.h>
-#include <ctype.h>
-#include <limits.h>
+#include <cygwin/kd.h>
 #include "cygerrno.h"
 #include "security.h"
 #include "path.h"
@@ -27,8 +23,9 @@ details. */
 #include "shared_info.h"
 #include "cygserver.h"
 #include "cygthread.h"
+#include "child_info.h"
 
-/* Tty master stuff */
+/* tty master stuff */
 
 fhandler_tty_master NO_COPY *tty_master;
 
@@ -60,7 +57,6 @@ fhandler_tty_master::set_winsize (bool sendSIGWINCH)
 int
 fhandler_tty_master::init ()
 {
-  slave = dev ();
   termios_printf ("Creating master for tty%d", get_unit ());
 
   if (init_console ())
@@ -69,27 +65,23 @@ fhandler_tty_master::init ()
       return -1;
     }
 
-  termios ti;
-  memset (&ti, 0, sizeof (ti));
-  console->tcsetattr (0, &ti);
-
-  cygwin_shared->tty[get_unit ()]->common_init (this);
+  if (!setup (false))
+    return 1;
 
   set_winsize (false);
 
-  inuse = get_ttyp ()->create_inuse (TTY_MASTER_ALIVE);
   set_close_on_exec (true);
 
   cygthread *h;
-  h = new cygthread (process_input, cygself, "ttyin");
+  h = new cygthread (process_input, 0, cygself, "ttyin");
   h->SetThreadPriority (THREAD_PRIORITY_HIGHEST);
   h->zap_h ();
 
-  h = new cygthread (process_ioctl, cygself, "ttyioctl");
+  h = new cygthread (process_ioctl, 0, cygself, "ttyioctl");
   h->SetThreadPriority (THREAD_PRIORITY_HIGHEST);
   h->zap_h ();
 
-  h = new cygthread (process_output, cygself, "ttyout");
+  h = new cygthread (process_output, 0, cygself, "ttyout");
   h->SetThreadPriority (THREAD_PRIORITY_HIGHEST);
   h->zap_h ();
 
@@ -112,13 +104,13 @@ DWORD
 fhandler_tty_common::__acquire_output_mutex (const char *fn, int ln,
 					   DWORD ms)
 {
-  if (strace.active)
+  if (strace.active ())
     strace.prntf (_STRACE_TERMIOS, fn, "(%d): tty output_mutex: waiting %d ms", ln, ms);
   DWORD res = WaitForSingleObject (output_mutex, ms);
   if (res == WAIT_OBJECT_0)
     {
 #ifndef DEBUGGING
-      if (strace.active)
+      if (strace.active ())
 	strace.prntf (_STRACE_TERMIOS, fn, "(%d): tty output_mutex: acquired", ln, res);
 #else
       ostack[osi].fn = fn;
@@ -137,7 +129,7 @@ fhandler_tty_common::__release_output_mutex (const char *fn, int ln)
   if (ReleaseMutex (output_mutex))
     {
 #ifndef DEBUGGING
-      if (strace.active)
+      if (strace.active ())
 	strace.prntf (_STRACE_TERMIOS, fn, "(%d): tty output_mutex released", ln);
 #else
       if (osi > 0)
@@ -147,6 +139,13 @@ fhandler_tty_common::__release_output_mutex (const char *fn, int ln)
       ostack[osi].ln = -ln;
 #endif
     }
+#ifdef DEBUGGING
+  else if (osi > 0)
+    {
+      system_printf ("couldn't release output mutex but we seem to own it, %E");
+      try_to_debug ();
+    }
+#endif
 }
 
 /* Process tty input. */
@@ -155,8 +154,8 @@ void
 fhandler_pty_master::doecho (const void *str, DWORD len)
 {
   acquire_output_mutex (INFINITE);
-  if (!WriteFile (get_ttyp ()->to_master, str, len, &len, NULL))
-    termios_printf ("Write to %p failed, %E", get_ttyp ()->to_master);
+  if (!WriteFile (to_master, str, len, &len, NULL))
+    termios_printf ("Write to %p failed, %E", to_master);
 //  WaitForSingleObject (output_done_event, INFINITE);
   release_output_mutex ();
 }
@@ -167,7 +166,7 @@ fhandler_pty_master::accept_input ()
   DWORD bytes_left;
   int ret = 1;
 
-  (void) WaitForSingleObject (input_mutex, INFINITE);
+  WaitForSingleObject (input_mutex, INFINITE);
 
   bytes_left = eat_readahead (-1);
 
@@ -427,9 +426,12 @@ process_ioctl (void *)
     {
       WaitForSingleObject (tty_master->ioctl_request_event, INFINITE);
       termios_printf ("ioctl() request");
-      tty_master->get_ttyp ()->ioctl_retval =
-      tty_master->console->ioctl (tty_master->get_ttyp ()->cmd,
-			     (void *) &tty_master->get_ttyp ()->arg);
+      tty *ttyp = tty_master->get_ttyp ();
+      ttyp->ioctl_retval =
+      tty_master->console->ioctl (ttyp->cmd,
+				  (ttyp->cmd == KDSKBMETA)
+				  ? (void *) ttyp->arg.value
+				  : (void *) &ttyp->arg);
       SetEvent (tty_master->ioctl_done_event);
     }
 }
@@ -438,7 +440,7 @@ process_ioctl (void *)
 /* Tty slave stuff */
 
 fhandler_tty_slave::fhandler_tty_slave ()
-  : fhandler_tty_common ()
+  : fhandler_tty_common (), inuse (NULL)
 {
   uninterruptible_io (true);
 }
@@ -448,26 +450,39 @@ fhandler_tty_slave::fhandler_tty_slave ()
 int
 fhandler_tty_slave::open (int flags, mode_t)
 {
+  HANDLE tty_owner, from_master_local, to_master_local;
+  HANDLE *handles[] =
+  {
+    &from_master_local, &input_available_event, &input_mutex, &inuse,
+    &ioctl_done_event, &ioctl_request_event, &output_done_event,
+    &output_mutex, &to_master_local, &tty_owner,
+    NULL
+  };
+
+  const char *errmsg = NULL;
+
+  for (HANDLE **h = handles; *h; h++)
+    **h = NULL;
+
   if (get_device () == FH_TTY)
-    pc.dev.tty_to_real_device ();
-  fhandler_tty_slave *arch = (fhandler_tty_slave *)
-    cygheap->fdtab.find_archetype (pc.dev);
+    dev().tty_to_real_device ();
+  fhandler_tty_slave *arch = (fhandler_tty_slave *) cygheap->fdtab.find_archetype (pc.dev);
   if (arch)
     {
       *this = *(fhandler_tty_slave *) arch;
-      termios_printf ("copied tty fhandler archetype");
+      termios_printf ("copied fhandler_tty_slave archetype");
       set_flags ((flags & ~O_TEXT) | O_BINARY);
-      cygheap->open_fhs++;
+      cygheap->manage_console_count ("fhandler_tty_slave::open<arch>", 1);
       goto out;
     }
 
   tcinit (cygwin_shared->tty[get_unit ()]);
 
-  attach_tty (get_unit ());
+  cygwin_shared->tty.attach (get_unit ());
 
   set_flags ((flags & ~O_TEXT) | O_BINARY);
   /* Create synchronisation events */
-  char buf[CYG_MAX_PATH];
+  char buf[MAX_PATH];
 
   /* output_done_event may or may not exist.  It will exist if the tty
      was opened by fhandler_tty_master::init, normally called at
@@ -479,22 +494,19 @@ fhandler_tty_slave::open (int flags, mode_t)
 
   if (!(output_mutex = get_ttyp ()->open_output_mutex ()))
     {
-      termios_printf ("open output mutex failed, %E");
-      __seterrno ();
-      return 0;
+      errmsg = "open output mutex failed, %E";
+      goto err;
     }
   if (!(input_mutex = get_ttyp ()->open_input_mutex ()))
     {
-      termios_printf ("open input mutex failed, %E");
-      __seterrno ();
-      return 0;
+      errmsg = "open input mutex failed, %E";
+      goto err;
     }
   shared_name (buf, INPUT_AVAILABLE_EVENT, get_unit ());
   if (!(input_available_event = OpenEvent (EVENT_ALL_ACCESS, TRUE, buf)))
     {
-      termios_printf ("open input event failed, %E");
-      __seterrno ();
-      return 0;
+      errmsg = "open input event failed, %E";
+      goto err;
     }
 
   /* The ioctl events may or may not exist.  See output_done_event,
@@ -512,111 +524,124 @@ fhandler_tty_slave::open (int flags, mode_t)
     release_output_mutex ();
   }
 
-  /* Duplicate tty handles.  */
-
-  if (!get_ttyp ()->from_slave || !get_ttyp ()->to_slave)
+  if (!get_ttyp ()->from_master || !get_ttyp ()->to_master)
     {
-      termios_printf ("tty handles have been closed");
+      errmsg = "tty handles have been closed";
       set_errno (EACCES);
-      return 0;
+      goto err_no_errno;
     }
 
-  HANDLE from_master_local;
-  HANDLE to_master_local;
-  from_master_local = to_master_local = NULL;
-
-#ifdef USE_SERVER
-  if (!wincap.has_security ()
+  if (myself->pid == get_ttyp ()->master_pid
       || cygserver_running == CYGSERVER_UNAVAIL
       || !cygserver_attach_tty (&from_master_local, &to_master_local))
-#endif
     {
-#ifdef USE_SERVER
-      termios_printf ("cannot dup handles via server. using old method.");
-#endif
-      HANDLE tty_owner = OpenProcess (PROCESS_DUP_HANDLE, FALSE,
-				      get_ttyp ()->master_pid);
-      termios_printf ("tty own handle %p",tty_owner);
-      if (tty_owner == NULL)
+      if (get_ttyp ()->master_pid < 0)
 	{
-	  termios_printf ("can't open tty (%d) handle process %d",
-			  get_unit (), get_ttyp ()->master_pid);
-	  __seterrno ();
-	  return 0;
+	  errmsg = "*** master is closed";
+	  set_errno (EAGAIN);
+	  goto err_no_errno;
+	}
+      pinfo p (get_ttyp ()->master_pid);
+      if (!p)
+	{
+	  errmsg = "*** couldn't find tty master";
+	  set_errno (EAGAIN);
+	  goto err_no_errno;
+	}
+      HANDLE tty_owner;
+      if (myself->pid == get_ttyp ()->master_pid)
+	{
+	  /* This is the most common case, just calling openpty. */
+	  termios_printf ("dup handles within myself.");
+	  tty_owner = GetCurrentProcess ();
+	}
+      else
+	{
+	  termios_printf ("cannot dup handles via server. using old method.");
+	  tty_owner = OpenProcess (PROCESS_DUP_HANDLE, FALSE, p->dwProcessId);
+	  if (tty_owner == NULL)
+	    {
+	      termios_printf ("can't open tty (%d) handle process %d",
+			      get_unit (), get_ttyp ()->master_pid);
+	      __seterrno ();
+	      goto err_no_msg;
+	    }
 	}
 
       if (!DuplicateHandle (tty_owner, get_ttyp ()->from_master,
-			    hMainProc, &from_master_local, 0, TRUE,
+			    GetCurrentProcess (), &from_master_local, 0, TRUE,
 			    DUPLICATE_SAME_ACCESS))
 	{
-	  termios_printf ("can't duplicate input, %E");
+	  termios_printf ("can't duplicate input from %u/%p, %E",
+			  get_ttyp ()->master_pid, get_ttyp ()->from_master);
 	  __seterrno ();
-	  return 0;
+	  goto err_no_msg;
 	}
 
       VerifyHandle (from_master_local);
       if (!DuplicateHandle (tty_owner, get_ttyp ()->to_master,
-			  hMainProc, &to_master_local, 0, TRUE,
+			  GetCurrentProcess (), &to_master_local, 0, TRUE,
 			  DUPLICATE_SAME_ACCESS))
 	{
-	  termios_printf ("can't duplicate output, %E");
-	  __seterrno ();
-	  return 0;
+	  errmsg = "can't duplicate output, %E";
+	  goto err;
 	}
       VerifyHandle (to_master_local);
-      CloseHandle (tty_owner);
+      if (tty_owner != GetCurrentProcess ())
+	CloseHandle (tty_owner);
     }
 
   termios_printf ("duplicated from_master %p->%p from tty_owner",
-      get_ttyp ()->from_master, from_master_local);
+		  get_ttyp ()->from_master, from_master_local);
   termios_printf ("duplicated to_master %p->%p from tty_owner",
-      get_ttyp ()->to_master, to_master_local);
+		  get_ttyp ()->to_master, to_master_local);
 
   set_io_handle (from_master_local);
   set_output_handle (to_master_local);
+  set_close_on_exec (!!(flags & O_CLOEXEC));
 
   set_open_status ();
-  if (cygheap->open_fhs++ == 0 && !GetConsoleCP () && !output_done_event
-      && wincap.pty_needs_alloc_console () && !GetProcessWindowStation ())
-    {
-      BOOL b;
-      HWINSTA h = CreateWindowStation (NULL, 0, GENERIC_READ | GENERIC_WRITE, &sec_none_nih);
-      termios_printf ("CreateWindowStation %p, %E", h);
-      if (h)
-	{
-	  b = SetProcessWindowStation (h);
-	  termios_printf ("SetProcessWindowStation %d, %E", b);
-	}
-      b = AllocConsole ();	// will cause flashing if workstation
-				// stuff fails
-      termios_printf ("%d = AllocConsole (), %E", b);
-      if (b)
-	init_console_handler ();
-    }
+  if (cygheap->manage_console_count ("fhandler_tty_slave::open", 1) == 1
+      && !output_done_event)
+    fhandler_console::need_invisible ();
 
   // FIXME: Do this better someday
-  arch = (fhandler_tty_slave *) cmalloc (HEAP_ARCHETYPES, sizeof (*this));
+  arch = (fhandler_tty_slave *) cmalloc_abort (HEAP_ARCHETYPES, sizeof (*this));
   *((fhandler_tty_slave **) cygheap->fdtab.add_archetype ()) = arch;
   archetype = arch;
   *arch = *this;
 
 out:
   usecount = 0;
-  archetype->usecount++;
-  report_tty_counts (this, "opened", "incremented ", "");
+  arch->usecount++;
+  report_tty_counts (this, "opened", "");
   myself->set_ctty (get_ttyp (), flags, arch);
 
   return 1;
+
+err:
+  __seterrno ();
+err_no_errno:
+  termios_printf (errmsg);
+err_no_msg:
+  for (HANDLE **h = handles; *h; h++)
+    if (**h && **h != INVALID_HANDLE_VALUE)
+      CloseHandle (**h);
+  return 0;
 }
 
 int
 fhandler_tty_slave::close ()
 {
-  if (!--cygheap->open_fhs && myself->ctty == -1)
-    FreeConsole ();
+  /* This used to always call fhandler_tty_common::close when hExeced but that
+     caused multiple closes of the handles associated with this tty.  Since
+     close_all_files is not called until after the cygwin process has synced
+     or before a non-cygwin process has exited, it should be safe to just
+     close this normally.  cgf 2006-05-20 */
+  cygheap->manage_console_count ("fhandler_tty_slave::close", -1);
 
   archetype->usecount--;
-  report_tty_counts (this, "closed", "decremented ", "");
+  report_tty_counts (this, "closed", "");
 
   if (archetype->usecount)
     {
@@ -629,6 +654,8 @@ fhandler_tty_slave::close ()
     }
 
   termios_printf ("closing last open %s handle", ttyname ());
+  if (inuse && !CloseHandle (inuse))
+    termios_printf ("CloseHandle (inuse), %E");
   return fhandler_tty_common::close ();
 }
 
@@ -636,13 +663,14 @@ int
 fhandler_tty_slave::cygserver_attach_tty (LPHANDLE from_master_ptr,
 					  LPHANDLE to_master_ptr)
 {
-#ifndef USE_SERVER
-  return 0;
-#else
   if (!from_master_ptr || !to_master_ptr)
     return 0;
 
-  client_request_attach_tty req ((DWORD) get_ttyp ()->master_pid,
+  pinfo p (get_ttyp ()->master_pid);
+  if (!p)
+    return 0;
+
+  client_request_attach_tty req (p->dwProcessId,
 				 (HANDLE) get_ttyp ()->from_master,
 				 (HANDLE) get_ttyp ()->to_master);
 
@@ -653,10 +681,9 @@ fhandler_tty_slave::cygserver_attach_tty (LPHANDLE from_master_ptr,
   *to_master_ptr = req.to_master ();
 
   return 1;
-#endif
 }
 
-void
+int
 fhandler_tty_slave::init (HANDLE, DWORD a, mode_t)
 {
   int flags = 0;
@@ -669,10 +696,10 @@ fhandler_tty_slave::init (HANDLE, DWORD a, mode_t)
   if (a == (GENERIC_READ | GENERIC_WRITE))
     flags = O_RDWR;
 
-  open (flags);
+  return open (flags);
 }
 
-int
+ssize_t __stdcall
 fhandler_tty_slave::write (const void *ptr, size_t len)
 {
   DWORD n, towrite = len;
@@ -771,7 +798,10 @@ fhandler_tty_slave::read (void *ptr, size_t& len)
       rc = WaitForMultipleObjects (2, w4, FALSE, waiter);
 
       if (rc == WAIT_TIMEOUT)
-	break;
+	{
+	  termios_printf ("wait timed out, waiter %u", waiter);
+	  break;
+	}
 
       if (rc == WAIT_FAILED)
 	{
@@ -812,7 +842,7 @@ fhandler_tty_slave::read (void *ptr, size_t& len)
       if (!ptr && len == UINT_MAX)
 	len = (size_t) bytes_in_pipe;
 
-      if (ptr && !vmin && !time_to_wait)
+      if (ptr && !bytes_in_pipe && !vmin && !time_to_wait)
 	{
 	  ReleaseMutex (input_mutex);
 	  len = (size_t) bytes_in_pipe;
@@ -906,7 +936,6 @@ fhandler_tty_slave::read (void *ptr, size_t& len)
     }
   termios_printf ("%d=read(%x, %d)", totalread, ptr, len);
   len = (size_t) totalread;
-  return;
 }
 
 int
@@ -914,108 +943,24 @@ fhandler_tty_slave::dup (fhandler_base *child)
 {
   fhandler_tty_slave *arch = (fhandler_tty_slave *) archetype;
   *(fhandler_tty_slave *) child = *arch;
+  child->set_flags (get_flags ());
   child->usecount = 0;
   arch->usecount++;
-  cygheap->open_fhs++;
-  report_tty_counts (child, "duped", "incremented ", "");
-  myself->set_ctty (get_ttyp (), openflags, arch);
+  cygheap->manage_console_count ("fhandler_tty_slave::dup", 1);
+  report_tty_counts (child, "duped", "");
   return 0;
 }
 
 int
-fhandler_tty_common::dup (fhandler_base *child)
+fhandler_pty_master::dup (fhandler_base *child)
 {
-  fhandler_tty_slave *fts = (fhandler_tty_slave *) child;
-  int errind;
-
-  fts->tcinit (get_ttyp ());
-
-  attach_tty (get_unit ());
-
-  HANDLE nh;
-
-  if (output_done_event == NULL)
-    fts->output_done_event = NULL;
-  else if (!DuplicateHandle (hMainProc, output_done_event, hMainProc,
-			     &fts->output_done_event, 0, 1,
-			     DUPLICATE_SAME_ACCESS))
-    {
-      errind = 1;
-      goto err;
-    }
-  if (ioctl_request_event == NULL)
-    fts->ioctl_request_event = NULL;
-  else if (!DuplicateHandle (hMainProc, ioctl_request_event, hMainProc,
-			     &fts->ioctl_request_event, 0, 1,
-			     DUPLICATE_SAME_ACCESS))
-    {
-      errind = 2;
-      goto err;
-    }
-  if (ioctl_done_event == NULL)
-    fts->ioctl_done_event = NULL;
-  else if (!DuplicateHandle (hMainProc, ioctl_done_event, hMainProc,
-			     &fts->ioctl_done_event, 0, 1,
-			     DUPLICATE_SAME_ACCESS))
-    {
-      errind = 3;
-      goto err;
-    }
-  if (!DuplicateHandle (hMainProc, input_available_event, hMainProc,
-			&fts->input_available_event, 0, 1,
-			DUPLICATE_SAME_ACCESS))
-    {
-      errind = 4;
-      goto err;
-    }
-  if (!DuplicateHandle (hMainProc, output_mutex, hMainProc,
-			&fts->output_mutex, 0, 1,
-			DUPLICATE_SAME_ACCESS))
-    {
-      errind = 5;
-      goto err;
-    }
-  if (!DuplicateHandle (hMainProc, input_mutex, hMainProc,
-			&fts->input_mutex, 0, 1,
-			DUPLICATE_SAME_ACCESS))
-    {
-      errind = 6;
-      goto err;
-    }
-  if (!DuplicateHandle (hMainProc, get_handle (), hMainProc,
-			&nh, 0, 1,
-			DUPLICATE_SAME_ACCESS))
-    {
-      errind = 7;
-      goto err;
-    }
-  fts->set_io_handle (nh);
-
-  if (!DuplicateHandle (hMainProc, get_output_handle (), hMainProc,
-			&nh, 0, 1,
-			DUPLICATE_SAME_ACCESS))
-    {
-      errind = 8;
-      goto err;
-    }
-  fts->set_output_handle (nh);
-
-  if (inuse == NULL)
-    fts->inuse = NULL;
-  else if (!DuplicateHandle (hMainProc, inuse, hMainProc,
-			     &fts->inuse, 0, 1,
-			     DUPLICATE_SAME_ACCESS))
-    {
-      errind = 9;
-      goto err;
-    }
-
+  fhandler_tty_master *arch = (fhandler_tty_master *) archetype;
+  *(fhandler_tty_master *) child = *arch;
+  child->set_flags (get_flags ());
+  child->usecount = 0;
+  arch->usecount++;
+  report_tty_counts (child, "duped master", "");
   return 0;
-
-err:
-  __seterrno ();
-  termios_printf ("dup %d failed in DuplicateHandle, %E", errind);
-  return -1;
 }
 
 int
@@ -1045,7 +990,7 @@ fhandler_tty_slave::tcflush (int queue)
     {
       size_t len = UINT_MAX;
       read (NULL, len);
-      ret = len >= 0;
+      ret = ((int) len) >= 0 ? 0 : -1;
     }
   if (queue == TCOFLUSH || queue == TCIOFLUSH)
     {
@@ -1065,8 +1010,8 @@ fhandler_tty_slave::ioctl (unsigned int cmd, void *arg)
       && myself->ctty == get_unit () && (get_ttyp ()->ti.c_lflag & TOSTOP))
     {
       /* background process */
-      termios_printf ("bg ioctl pgid %d, tpgid %d, ctty %d",
-		      myself->pgid, get_ttyp ()->getpgid (), myself->ctty);
+      termios_printf ("bg ioctl pgid %d, tpgid %d, %s", myself->pgid,
+		      get_ttyp ()->getpgid (), myctty ());
       raise (SIGTTOU);
     }
 
@@ -1076,6 +1021,8 @@ fhandler_tty_slave::ioctl (unsigned int cmd, void *arg)
     case TIOCGWINSZ:
     case TIOCSWINSZ:
     case TIOCLINUX:
+    case KDGKBMETA:
+    case KDSKBMETA:
       break;
     case FIONBIO:
       set_nonblocking (*(int *) arg);
@@ -1090,6 +1037,7 @@ fhandler_tty_slave::ioctl (unsigned int cmd, void *arg)
 
   get_ttyp ()->cmd = cmd;
   get_ttyp ()->ioctl_retval = 0;
+  int val;
   switch (cmd)
     {
     case TIOCGWINSZ:
@@ -1121,7 +1069,7 @@ fhandler_tty_slave::ioctl (unsigned int cmd, void *arg)
 	}
       break;
     case TIOCLINUX:
-      int val = *(unsigned char *) arg;
+      val = *(unsigned char *) arg;
       if (val != 6 || !ioctl_request_event || !ioctl_done_event)
 	  get_ttyp ()->ioctl_retval = -EINVAL;
       else
@@ -1129,9 +1077,31 @@ fhandler_tty_slave::ioctl (unsigned int cmd, void *arg)
 	  get_ttyp ()->arg.value = val;
 	  SetEvent (ioctl_request_event);
 	  WaitForSingleObject (ioctl_done_event, INFINITE);
-	  *(unsigned char *) arg = get_ttyp ()->arg.value & 0xFF;
+	  *(unsigned char *) arg = (unsigned char) (get_ttyp ()->arg.value);
 	}
       break;
+    case KDGKBMETA:
+      if (ioctl_request_event)
+	{
+	  SetEvent (ioctl_request_event);
+	  if (ioctl_done_event)
+	    WaitForSingleObject (ioctl_done_event, INFINITE);
+	  *(int *) arg = get_ttyp ()->arg.value;
+	}
+      else
+	get_ttyp ()->ioctl_retval = -EINVAL;
+      break;
+    case KDSKBMETA:
+      if (ioctl_request_event)
+	{
+	  get_ttyp ()->arg.value = (int) arg;
+	  SetEvent (ioctl_request_event);
+	  if (ioctl_done_event)
+	    WaitForSingleObject (ioctl_done_event, INFINITE);
+	}
+      else
+	get_ttyp ()->ioctl_retval = -EINVAL;
+      break;
     }
 
   release_output_mutex ();
@@ -1151,29 +1121,50 @@ out:
  fhandler_pty_master
 */
 fhandler_pty_master::fhandler_pty_master ()
-  : fhandler_tty_common ()
+  : fhandler_tty_common (), pktmode (0), need_nl (0), dwProcessId (0)
 {
 }
 
 int
 fhandler_pty_master::open (int flags, mode_t)
 {
-  int ntty = cygwin_shared->tty.allocate_tty (false);
+  int ntty;
+  ntty = cygwin_shared->tty.allocate (false);
   if (ntty < 0)
     return 0;
 
-  slave = *ttys_dev;
-  slave.setunit (ntty);
-  cygwin_shared->tty[ntty]->common_init (this);
-  ReleaseMutex (tty_mutex);	// lock was set in allocate_tty
-  inuse = get_ttyp ()->create_inuse (TTY_MASTER_ALIVE);
+  dev().devn = FHDEV (DEV_TTYM_MAJOR, ntty);
+  if (!setup (true))
+    {
+      lock_ttys::release ();
+      return 0;
+    }
+  lock_ttys::release ();
   set_flags ((flags & ~O_TEXT) | O_BINARY);
   set_open_status ();
+  //
+  // FIXME: Do this better someday
+  fhandler_pty_master *arch = (fhandler_tty_master *) cmalloc_abort (HEAP_ARCHETYPES, sizeof (*this));
+  *((fhandler_pty_master **) cygheap->fdtab.add_archetype ()) = arch;
+  archetype = arch;
+  *arch = *this;
+  arch->dwProcessId = GetCurrentProcessId ();
 
-  termios_printf ("opened pty master tty%d", get_unit ());
+  usecount = 0;
+  arch->usecount++;
+  char buf[sizeof ("opened pty master for ttyNNNNNNNNNNN")];
+  __small_sprintf (buf, "opened pty master for tty%d", get_unit ());
+  report_tty_counts (this, buf, "");
   return 1;
 }
 
+_off64_t
+fhandler_tty_common::lseek (_off64_t, int)
+{
+  set_errno (ESPIPE);
+  return -1;
+}
+
 int
 fhandler_tty_common::close ()
 {
@@ -1184,29 +1175,18 @@ fhandler_tty_common::close ()
     termios_printf ("CloseHandle (ioctl_done_event), %E");
   if (ioctl_request_event && !CloseHandle (ioctl_request_event))
     termios_printf ("CloseHandle (ioctl_request_event), %E");
-  if (inuse && !CloseHandle (inuse))
-    termios_printf ("CloseHandle (inuse), %E");
   if (!ForceCloseHandle (input_mutex))
     termios_printf ("CloseHandle (input_mutex<%p>), %E", input_mutex);
   if (!ForceCloseHandle (output_mutex))
     termios_printf ("CloseHandle (output_mutex<%p>), %E", output_mutex);
-
-  /* Send EOF to slaves if master side is closed */
-  if (!get_ttyp ()->master_alive ())
-    {
-      termios_printf ("no more masters left. sending EOF");
-      SetEvent (input_available_event);
-    }
-
-  if (!ForceCloseHandle (input_available_event))
-    termios_printf ("CloseHandle (input_available_event<%p>), %E", input_available_event);
   if (!ForceCloseHandle1 (get_handle (), from_pty))
     termios_printf ("CloseHandle (get_handle ()<%p>), %E", get_handle ());
   if (!ForceCloseHandle1 (get_output_handle (), to_pty))
     termios_printf ("CloseHandle (get_output_handle ()<%p>), %E", get_output_handle ());
 
-  inuse = NULL;
-  set_io_handle (NULL);
+  if (!ForceCloseHandle (input_available_event))
+    termios_printf ("CloseHandle (input_available_event<%p>), %E", input_available_event);
+
   return 0;
 }
 
@@ -1217,28 +1197,40 @@ fhandler_pty_master::close ()
   while (accept_input () > 0)
     continue;
 #endif
+  archetype->usecount--;
+  report_tty_counts (this, "closing master", "");
+
+  if (archetype->usecount)
+    {
+#ifdef DEBUGGING
+      if (archetype->usecount < 0)
+	system_printf ("error: usecount %d", archetype->usecount);
+#endif
+      termios_printf ("just returning because archetype usecount is != 0");
+      return 0;
+    }
+
+  fhandler_tty_master *arch = (fhandler_tty_master *) archetype;
+  termios_printf ("closing from_master(%p)/to_master(%p) since we own them(%d)",
+		  arch->from_master, arch->to_master, arch->dwProcessId);
+  if (cygwin_finished_initializing)
+    {
+      if (!ForceCloseHandle (arch->from_master))
+	termios_printf ("error closing from_master %p, %E", arch->from_master);
+      if (!ForceCloseHandle (arch->to_master))
+	termios_printf ("error closing from_master %p, %E", arch->to_master);
+    }
   fhandler_tty_common::close ();
 
-  if (!get_ttyp ()->master_alive ())
-    {
-      termios_printf ("freeing tty%d (%d)", get_unit (), get_ttyp ()->ntty);
-#if 0
-      if (get_ttyp ()->to_slave)
-	ForceCloseHandle1 (get_ttyp ()->to_slave, to_slave);
-      if (get_ttyp ()->from_slave)
-	ForceCloseHandle1 (get_ttyp ()->from_slave, from_slave);
-#endif
-      if (get_ttyp ()->from_master)
-	CloseHandle (get_ttyp ()->from_master);
-      if (get_ttyp ()->to_master)
-	CloseHandle (get_ttyp ()->to_master);
-      get_ttyp ()->init ();
-    }
+  if (hExeced || get_ttyp ()->master_pid != myself->pid)
+    termios_printf ("not clearing: %d, master_pid %d", hExeced, get_ttyp ()->master_pid);
+  else
+    get_ttyp ()->set_master_closed ();
 
   return 0;
 }
 
-int
+ssize_t __stdcall
 fhandler_pty_master::write (const void *ptr, size_t len)
 {
   int i;
@@ -1262,7 +1254,6 @@ void __stdcall
 fhandler_pty_master::read (void *ptr, size_t& len)
 {
   len = (size_t) process_slave_output ((char *) ptr, len, pktmode);
-  return;
 }
 
 int
@@ -1329,7 +1320,7 @@ fhandler_pty_master::ioctl (unsigned int cmd, void *arg)
 char *
 fhandler_pty_master::ptsname ()
 {
-  static char buf[32];
+  static char buf[TTY_NAME_MAX];
 
   __small_sprintf (buf, "/dev/tty%d", get_unit ());
   return buf;
@@ -1338,73 +1329,23 @@ fhandler_pty_master::ptsname ()
 void
 fhandler_tty_common::set_close_on_exec (bool val)
 {
-  if (archetype)
-    close_on_exec (val);
-  else
-    {
-      if (output_done_event)
-	set_no_inheritance (output_done_event, val);
-      if (ioctl_request_event)
-	set_no_inheritance (ioctl_request_event, val);
-      if (ioctl_done_event)
-	set_no_inheritance (ioctl_done_event, val);
-      if (inuse)
-	set_no_inheritance (inuse, val);
-      set_no_inheritance (output_mutex, val);
-      set_no_inheritance (input_mutex, val);
-      set_no_inheritance (input_available_event, val);
-      set_no_inheritance (output_handle, val);
-#ifndef DEBUGGING
-      fhandler_base::set_close_on_exec (val);
-#else
-      /* FIXME: This is a duplication from fhandler_base::set_close_on_exec.
-	 It is here because we need to specify the "from_pty" stuff here or
-	 we'll get warnings from ForceCloseHandle when debugging. */
-      set_no_inheritance (get_io_handle (), val);
-      close_on_exec (val);
-#endif
-    }
+  // Cygwin processes will handle this specially on exec.
+  close_on_exec (val);
 }
 
 void
 fhandler_tty_slave::fixup_after_fork (HANDLE parent)
 {
+  // fork_fixup (parent, inuse, "inuse");
   // fhandler_tty_common::fixup_after_fork (parent);
-  report_tty_counts (this, "inherited", "", "");
+  report_tty_counts (this, "inherited", "");
 }
 
 void
-fhandler_tty_common::fixup_after_fork (HANDLE parent)
+fhandler_tty_slave::fixup_after_exec ()
 {
-  fhandler_termios::fixup_after_fork (parent);
-  if (output_done_event)
-    fork_fixup (parent, output_done_event, "output_done_event");
-  if (ioctl_request_event)
-    fork_fixup (parent, ioctl_request_event, "ioctl_request_event");
-  if (ioctl_done_event)
-    fork_fixup (parent, ioctl_done_event, "ioctl_done_event");
-  if (output_mutex)
-    fork_fixup (parent, output_mutex, "output_mutex");
-  if (input_mutex)
-    fork_fixup (parent, input_mutex, "input_mutex");
-  if (input_available_event)
-    fork_fixup (parent, input_available_event, "input_available_event");
-  fork_fixup (parent, inuse, "inuse");
-}
-
-void
-fhandler_pty_master::set_close_on_exec (bool val)
-{
-  fhandler_tty_common::set_close_on_exec (val);
-
-  /* FIXME: There is a console handle leak here. */
-  if (get_ttyp ()->master_pid == GetCurrentProcessId ())
-    {
-      get_ttyp ()->from_slave = get_handle ();
-      get_ttyp ()->to_slave = get_output_handle ();
-      termios_printf ("from_slave %p, to_slave %p", get_handle (),
-		      get_output_handle ());
-    }
+  if (!close_on_exec ())
+    fixup_after_fork (NULL);
 }
 
 int
@@ -1415,7 +1356,160 @@ fhandler_tty_master::init_console ()
     return -1;
 
   console->init (INVALID_HANDLE_VALUE, GENERIC_READ | GENERIC_WRITE, O_BINARY);
-  cygheap->open_fhs--;		/* handled when individual fds are opened */
+  cygheap->manage_console_count ("fhandler_tty_master::init_console", -1, true);
   console->uninterruptible_io (true);
   return 0;
 }
+
+#define close_maybe(h) \
+  do { \
+    if (h && h != INVALID_HANDLE_VALUE) \
+      CloseHandle (h); \
+  } while (0)
+
+bool
+fhandler_pty_master::setup (bool ispty)
+{
+  int res;
+  tty& t = *cygwin_shared->tty[get_unit ()];
+
+  tcinit (&t, true);		/* Set termios information.  Force initialization. */
+
+  const char *errstr = NULL;
+  DWORD pipe_mode = PIPE_NOWAIT;
+
+  /* Create communication pipes */
+
+  char pipename[sizeof("ttyNNNN-from-master")];
+  __small_sprintf (pipename, "tty%d-from-master", get_unit ());
+  res = fhandler_pipe::create_selectable (&sec_none_nih, from_master,
+					  get_output_handle (), 128 * 1024,
+					  pipename);
+  if (res)
+    {
+      errstr = "input pipe";
+      goto err;
+    }
+  /* Only ptys should create inheritable handles by default.  ttys are
+     parcelled out on an as-needed basis and handle inheritance differently. */
+  if (ispty && !SetHandleInformation (get_output_handle (), HANDLE_FLAG_INHERIT, HANDLE_FLAG_INHERIT))
+    {
+      errstr = "inheritable get_output_handle ()";
+      goto err;
+    }
+
+  if (!SetNamedPipeHandleState (get_output_handle (), &pipe_mode, NULL, NULL))
+    termios_printf ("can't set output_handle(%p) to non-blocking mode",
+		    get_output_handle ());
+
+  __small_sprintf (pipename, "tty%d-to-master", get_unit ());
+  res = fhandler_pipe::create_selectable (&sec_none_nih, get_io_handle (),
+					  to_master, 128 * 1024, pipename);
+  if (res)
+    {
+      errstr = "output pipe";
+      goto err;
+    }
+  /* Only ptys should create inheritable handles by default.  ttys are
+     parcelled out on an as-needed basis and handle inheritance differently. */
+  if (ispty && !SetHandleInformation (get_io_handle (), HANDLE_FLAG_INHERIT, HANDLE_FLAG_INHERIT))
+    {
+      errstr = "inheritable get_io_handle ()";
+      goto err;
+    }
+
+  need_nl = 0;
+
+  /* We do not allow others to open us (for handle duplication)
+     but rely on cygheap->inherited_ctty for descendant processes.
+     In the future the cygserver may allow access by others. */
+
+  if (cygserver_running == CYGSERVER_UNKNOWN)
+    cygserver_init ();
+
+  /* Create synchronisation events */
+
+  if (!ispty)
+    {
+      if (!(output_done_event = t.get_event (errstr = OUTPUT_DONE_EVENT)))
+	goto err;
+      if (!(ioctl_done_event = t.get_event (errstr = IOCTL_DONE_EVENT)))
+	goto err;
+      if (!(ioctl_request_event = t.get_event (errstr = IOCTL_REQUEST_EVENT)))
+	goto err;
+    }
+
+  if (!(input_available_event = t.get_event (errstr = INPUT_AVAILABLE_EVENT, TRUE)))
+    goto err;
+
+  char buf[MAX_PATH];
+  errstr = shared_name (buf, OUTPUT_MUTEX, t.ntty);
+  if (!(output_mutex = CreateMutex (&sec_all, FALSE, buf)))
+    goto err;
+
+  errstr = shared_name (buf, INPUT_MUTEX, t.ntty);
+  if (!(input_mutex = CreateMutex (&sec_all, FALSE, buf)))
+    goto err;
+
+  t.from_master = from_master;
+  t.to_master = to_master;
+  // /* screws up tty master */ ProtectHandle1INH (output_mutex, output_mutex);
+  // /* screws up tty master */ ProtectHandle1INH (input_mutex, input_mutex);
+  t.winsize.ws_col = 80;
+  t.winsize.ws_row = 25;
+  t.master_pid = myself->pid;
+
+  termios_printf ("tty%d opened - from_slave %p, to_slave %p", t.ntty,
+		  get_io_handle (), get_output_handle ());
+  return true;
+
+err:
+  __seterrno ();
+  close_maybe (get_io_handle ());
+  close_maybe (get_output_handle ());
+  close_maybe (output_done_event);
+  close_maybe (ioctl_done_event);
+  close_maybe (ioctl_request_event);
+  close_maybe (input_available_event);
+  close_maybe (output_mutex);
+  close_maybe (input_mutex);
+  close_maybe (from_master);
+  close_maybe (to_master);
+  termios_printf ("tty%d open failed - failed to create %s", errstr);
+  return false;
+}
+
+void
+fhandler_pty_master::fixup_after_fork (HANDLE parent)
+{
+  DWORD wpid = GetCurrentProcessId ();
+  fhandler_tty_master *arch = (fhandler_tty_master *) archetype;
+  if (arch->dwProcessId != wpid)
+    {
+      tty& t = *get_ttyp ();
+      if (!DuplicateHandle (parent, arch->from_master, GetCurrentProcess (),
+			    &arch->from_master, 0, false, DUPLICATE_SAME_ACCESS))
+	system_printf ("couldn't duplicate from_parent(%p), %E", arch->from_master);
+      if (!DuplicateHandle (parent, arch->to_master, GetCurrentProcess (),
+			    &arch->to_master, 0, false, DUPLICATE_SAME_ACCESS))
+	system_printf ("couldn't duplicate to_parent(%p), %E", arch->from_master);
+      if (myself->pid == t.master_pid)
+	{
+	  t.from_master = arch->from_master;
+	  t.to_master = arch->to_master;
+	}
+      arch->dwProcessId = wpid;
+    }
+  from_master = arch->from_master;
+  to_master = arch->to_master;
+  report_tty_counts (this, "inherited master", "");
+}
+
+void
+fhandler_pty_master::fixup_after_exec ()
+{
+  if (!close_on_exec ())
+    fixup_after_fork (spawn_info->parent);
+  else
+    from_master = to_master = NULL;
+}
