@@ -1,6 +1,6 @@
 /* hires.h: Definitions for hires clock calculations
 
-   Copyright 2002 Red Hat, Inc.
+   Copyright 2002, 2003, 2004, 2005, 2009 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -19,15 +19,12 @@ details. */
    The tv_sec argument in timeval structures cannot exceed
    HIRES_DELAY_MAX / 1000 - 1, so that adding fractional part
    and rounding won't exceed HIRES_DELAY_MAX */
-#define HIRES_DELAY_MAX (((UINT_MAX - 10000) / 1000) * 1000) + 10
+#define HIRES_DELAY_MAX ((((UINT_MAX - 10000) / 1000) * 1000) + 10)
 
 class hires_base
 {
  protected:
   int inited;
- public:
-  virtual LONGLONG usecs (bool justdelta) {return 0LL;}
-  virtual ~hires_base () {}
 };
 
 class hires_us : hires_base
@@ -42,15 +39,15 @@ class hires_us : hires_base
 
 class hires_ms : hires_base
 {
-  DWORD initime_ms;
-  LARGE_INTEGER initime_us;
-  static UINT minperiod;
-  UINT prime ();
+  LONGLONG initime_ns;
+  void prime ();
  public:
-  LONGLONG usecs (bool justdelta);
+  LONGLONG nsecs ();
+  LONGLONG usecs () {return nsecs () / 10LL;}
+  LONGLONG msecs () {return nsecs () / 10000LL;}
   UINT dmsecs () { return timeGetTime (); }
-  UINT resolution () { return minperiod ?: prime (); }
-
+  UINT resolution ();
+  LONGLONG uptime () {return (nsecs () - initime_ns) / 10000LL;}
 };
 
 extern hires_ms gtod;
