@@ -1,6 +1,6 @@
 /* types.h
 
-   Copyright 2001, 2002, 2003 Red Hat Inc.
+   Copyright 2001, 2002, 2003, 2005, 2006, 2010 Red Hat Inc.
    Written by Robert Collins <rbtcollins@hotmail.com>
 
 This file is part of Cygwin.
@@ -18,6 +18,8 @@ extern "C"
 #define _CYGWIN_TYPES_H
 
 #include <sys/sysmacros.h>
+#include <stdint.h>
+#include <endian.h>
 
 #ifndef __timespec_t_defined
 #define __timespec_t_defined
@@ -31,24 +33,16 @@ typedef struct timespec timestruc_t;
 
 #ifndef __off_t_defined
 #define __off_t_defined
-typedef long __off32_t;
-typedef long long __off64_t;
-#ifdef __CYGWIN_USE_BIG_TYPES__
-typedef __off64_t off_t;
-#else
-typedef __off32_t off_t;
-#endif
+typedef _off64_t off_t;
 #endif /*__off_t_defined*/
 
+typedef __loff_t loff_t;
+
 #ifndef __dev_t_defined
 #define __dev_t_defined
 typedef short __dev16_t;
 typedef unsigned long __dev32_t;
-#ifdef __CYGWIN_USE_BIG_TYPES__
 typedef __dev32_t dev_t;
-#else
-typedef __dev16_t dev_t;
-#endif
 #endif /*__dev_t_defined*/
 
 #ifndef __blksize_t_defined
@@ -60,90 +54,95 @@ typedef long blksize_t;
 #define __blkcnt_t_defined
 typedef long __blkcnt32_t;
 typedef long long __blkcnt64_t;
-#ifdef __CYGWIN_USE_BIG_TYPES__
 typedef __blkcnt64_t  blkcnt_t;
-#else
-typedef __blkcnt32_t  blkcnt_t;
-#endif
 #endif /*__blkcnt_t_defined*/
 
+#ifndef __fsblkcnt_t_defined
+#define __fsblkcnt_t_defined
+typedef unsigned long fsblkcnt_t;
+#endif /* __fsblkcnt_t_defined */
+
+#ifndef __fsfilcnt_t_defined
+#define __fsfilcnt_t_defined
+typedef unsigned long fsfilcnt_t;
+#endif /* __fsfilcnt_t_defined */
+
 #ifndef __uid_t_defined
 #define __uid_t_defined
 typedef unsigned short __uid16_t;
 typedef unsigned long  __uid32_t;
-#ifdef __CYGWIN_USE_BIG_TYPES__
 typedef __uid32_t uid_t;
-#else
-typedef __uid16_t uid_t;
-#endif
 #endif /*__uid_t_defined*/
 
 #ifndef __gid_t_defined
 #define __gid_t_defined
 typedef unsigned short __gid16_t;
 typedef unsigned long  __gid32_t;
-#ifdef __CYGWIN_USE_BIG_TYPES__
 typedef __gid32_t gid_t;
-#else
-typedef __gid16_t gid_t;
-#endif
 #endif /*__gid_t_defined*/
 
 #ifndef __ino_t_defined
 #define __ino_t_defined
-#ifdef __CYGWIN_USE_BIG_TYPES1__
-typedef unsigned long long ino_t;
-#else
-typedef unsigned long ino_t;
-#endif
+typedef unsigned long __ino32_t;
+typedef unsigned long long __ino64_t;
+typedef __ino64_t ino_t;
 #endif /*__ino_t_defined*/
 
+/* Generic ID type, must match at least pid_t, uid_t and gid_t in size. */
+#ifndef __id_t_defined
+#define __id_t_defined
+typedef unsigned long id_t;
+#endif /* __id_t_defined */
+
+#if defined (__INSIDE_CYGWIN__)
+struct __flock32 {
+	short	 l_type;	/* F_RDLCK, F_WRLCK, or F_UNLCK */
+	short	 l_whence;	/* flag to choose starting offset */
+	_off_t	 l_start;	/* relative offset, in bytes */
+	_off_t	 l_len;		/* length, in bytes; 0 means lock to EOF */
+	short	 l_pid;		/* returned with F_GETLK */
+	short	 l_xxx;		/* reserved for future use */
+};
+
+struct __flock64 {
+	short	 l_type;	/* F_RDLCK, F_WRLCK, or F_UNLCK */
+	short	 l_whence;	/* flag to choose starting offset */
+	_off64_t l_start;	/* relative offset, in bytes */
+	_off64_t l_len;		/* length, in bytes; 0 means lock to EOF */
+	pid_t	 l_pid;		/* returned with F_GETLK */
+};
+#endif
+
+struct flock {
+	short	 l_type;	/* F_RDLCK, F_WRLCK, or F_UNLCK */
+	short	 l_whence;	/* flag to choose starting offset */
+	off_t	 l_start;	/* relative offset, in bytes */
+	off_t	 l_len;		/* length, in bytes; 0 means lock to EOF */
+	pid_t	 l_pid;		/* returned with F_GETLK */
+};
+
+#ifndef __key_t_defined
+#define __key_t_defined
+typedef long long key_t;
+#endif /* __key_t_defined */
+
 #ifndef __BIT_TYPES_DEFINED
 #define __BIT_TYPES_DEFINED__ 1
 
-#ifndef __vm_offset_t
+#ifndef __vm_offset_t_defined
 #define __vm_offset_t_defined
 typedef unsigned long vm_offset_t;
 #endif /*__vm_offset_t_defined*/
 
-#ifndef __vm_size_t
+#ifndef __vm_size_t_defined
 #define __vm_size_t_defined
 typedef unsigned long vm_size_t;
 #endif /*__vm_size_t_defined*/
 
-#ifndef __int8_t_defined
-#define __int8_t_defined
-typedef char int8_t;
-#endif
-#ifndef __int16_t_defined
-#define __int16_t_defined
-typedef __int16_t int16_t;
-#endif
-#ifndef __int32_t_defined
-#define __int32_t_defined
-typedef __int32_t int32_t;
-#endif
-#ifndef __int64_t_defined
-#define __int64_t_defined
-typedef __int64_t int64_t;
-#endif
-
-#ifndef __uint8_t_defined
-#define __uint8_t_defined
-typedef unsigned char uint8_t;
-#endif
-#ifndef __uint16_t_defined
-#define __uint16_t_defined
-typedef __uint16_t uint16_t;
-#endif
-#ifndef __uint32_t_defined
-#define __uint32_t_defined
-typedef __uint32_t uint32_t;
-#endif
-#ifndef __uint64_t_defined
-#define __uint64_t_defined
-typedef __uint64_t uint64_t;
-#endif
+#ifndef __vm_object_t_defined
+#define __vm_object_t_defined
+typedef void *vm_object_t;
+#endif /* __vm_object_t_defined */
 
 #ifndef __u_int8_t_defined
 #define __u_int8_t_defined
@@ -162,16 +161,6 @@ typedef __uint32_t u_int32_t;
 typedef __uint64_t u_int64_t;
 #endif
 
-#ifndef __uintptr_t_defined
-#define __uintptr_t_defined
-typedef unsigned long uintptr_t;
-#endif
-
-#ifndef __intptr_t_defined
-#define __intptr_t_defined
-typedef long intptr_t;
-#endif
-
 #ifndef __register_t_defined
 #define __register_t_defined
 typedef __int32_t register_t;
