@@ -6,7 +6,8 @@
 
 #include "winsup.h"
 #include "cygerrno.h"
-#include <windows.h>
+#include "sync.h"
+#define STD_INSPIRED
 #define lint
 
 #define USG_COMPAT
@@ -93,14 +94,11 @@ static char	privatehid[] = "@(#)private.h	7.48";
 ** Nested includes
 */
 
-#include "sys/types.h"	/* for time_t */
 #include "stdio.h"
 #include "limits.h"	/* for CHAR_BIT */
-#include "time.h"
 #include "stdlib.h"
 
 #if HAVE_GETTEXT - 0
-#include "libintl.h"
 #endif /* HAVE_GETTEXT - 0 */
 
 #if HAVE_UNISTD_H - 0
@@ -170,7 +168,6 @@ static char	privatehid[] = "@(#)private.h	7.48";
 
 #ifndef MAXPATHLEN
 #ifdef unix
-#include "sys/param.h"
 #endif /* defined unix */
 #endif /* !defined MAXPATHLEN */
 
@@ -629,12 +626,12 @@ static struct tm	tm;
 #undef _daylight
 
 #ifdef USG_COMPAT
-time_t			timezone;
-int			daylight;
+long		timezone;	/* was time_t but POSIX requires long. */
+int		daylight;
 #endif /* defined USG_COMPAT */
 
 #ifdef ALTZONE
-time_t			altzone;
+time_t		altzone;
 #endif /* defined ALTZONE */
 
 static long
@@ -730,9 +727,9 @@ tzload(const char *name, struct state *sp)
 				return -1;
 			if ((strlen(p) + strlen(name) + 1) >= sizeof fullname)
 				return -1;
-			(void) strcpy(fullname, p);
-			(void) strcat(fullname, "/");
-			(void) strcat(fullname, name);
+			strcpy(fullname, p);
+			strcat(fullname, "/");
+			strcat(fullname, name);
 			/*
 			** Set doaccess if '.' (as in "../") shows up in name.
 			*/
@@ -862,6 +859,10 @@ tzload(const char *name, struct state *sp)
 			}
 		}
 	}
+	__gettzinfo ()->__tzrule[0].offset
+				= -sp->ttis[1].tt_gmtoff;
+	__gettzinfo ()->__tzrule[1].offset
+				= -sp->ttis[0].tt_gmtoff;
 	return 0;
 }
 
@@ -1235,6 +1236,10 @@ tzparse(const char *name, struct state *sp, const int lastditch)
 				janfirst += year_lengths[isleap(year)] *
 					SECSPERDAY;
 			}
+			__gettzinfo ()->__tzrule[0].offset
+						= -sp->ttis[1].tt_gmtoff;
+			__gettzinfo ()->__tzrule[1].offset
+						= -sp->ttis[0].tt_gmtoff;
 		} else {
 			register long	theirstdoffset;
 			register long	theirdstoffset;
@@ -1321,6 +1326,10 @@ tzparse(const char *name, struct state *sp, const int lastditch)
 			sp->ttis[1].tt_isdst = true;
 			sp->ttis[1].tt_abbrind = stdlen + 1;
 			sp->typecnt = 2;
+			__gettzinfo ()->__tzrule[0].offset
+						= -sp->ttis[0].tt_gmtoff;
+			__gettzinfo ()->__tzrule[1].offset
+						= -sp->ttis[1].tt_gmtoff;
 		}
 	} else {
 		dstlen = 0;
@@ -1329,6 +1338,8 @@ tzparse(const char *name, struct state *sp, const int lastditch)
 		sp->ttis[0].tt_gmtoff = -stdoffset;
 		sp->ttis[0].tt_isdst = 0;
 		sp->ttis[0].tt_abbrind = 0;
+		__gettzinfo ()->__tzrule[0].offset = -sp->ttis[0].tt_gmtoff;
+		__gettzinfo ()->__tzrule[1].offset = -sp->ttis[0].tt_gmtoff;
 	}
 	sp->charcnt = stdlen + 1;
 	if (dstlen != 0)
@@ -1336,11 +1347,11 @@ tzparse(const char *name, struct state *sp, const int lastditch)
 	if ((size_t) sp->charcnt > sizeof sp->chars)
 		return -1;
 	cp = sp->chars;
-	(void) strncpy(cp, stdname, stdlen);
+	strncpy(cp, stdname, stdlen);
 	cp += stdlen;
 	*cp++ = '\0';
 	if (dstlen != 0) {
-		(void) strncpy(cp, dstname, dstlen);
+		strncpy(cp, dstname, dstlen);
 		*(cp + dstlen) = '\0';
 	}
 	return 0;
@@ -1350,7 +1361,7 @@ static void
 gmtload(struct state *sp)
 {
 	if (tzload(gmt, sp) != 0)
-		(void) tzparse(gmt, sp, true);
+		tzparse(gmt, sp, true);
 }
 
 #ifndef STD_INSPIRED
@@ -1445,7 +1456,11 @@ tzsetwall P((void))
 		settzname();
 		lcl_is_set = 1;
 		strlcpy(lcl_TZname, buf, sizeof (lcl_TZname));
+#if 0
+		/* Huh?  POSIX doesn't mention anywhere that tzset should
+		   set $TZ.  That's not right. */
 		setenv("TZ", lcl_TZname, 1);
+#endif
 		return;
 	    }
 	}
@@ -1455,28 +1470,32 @@ tzsetwall P((void))
 	settzname();
 }
 
+static NO_COPY muto tzset_guard;
+
 extern "C" void
 tzset P((void))
 {
+	tzset_guard.init ("tzset_guard")->acquire ();
 	const char *	name = getenv("TZ");
 
 	if (name == NULL) {
-		tzsetwall();
-		return;
+		if (!lcl_is_set)
+			tzsetwall();
+		goto out;
 	}
 
 	if (lcl_is_set > 0  &&  strcmp(lcl_TZname, name) == 0)
-		return;
+		goto out;
 	lcl_is_set = (strlen(name) < sizeof (lcl_TZname));
 	if (lcl_is_set)
-		(void) strcpy(lcl_TZname, name);
+		strcpy(lcl_TZname, name);
 
 #ifdef ALL_STATE
 	if (lclptr == NULL) {
 		lclptr = (struct state *) malloc(sizeof *lclptr);
 		if (lclptr == NULL) {
 			settzname();	/* all we can do */
-			return;
+			goto out;
 		}
 	}
 #endif /* defined ALL_STATE */
@@ -1488,12 +1507,14 @@ tzset P((void))
 		lclptr->timecnt = 0;
 		lclptr->ttis[0].tt_gmtoff = 0;
 		lclptr->ttis[0].tt_abbrind = 0;
-		(void) strcpy(lclptr->chars, gmt);
+		strcpy(lclptr->chars, gmt);
 	} else if (tzload(name, lclptr) != 0) {
 		if (name[0] == ':' || tzparse(name, lclptr, false) != 0)
-			(void) gmtload(lclptr);
+			gmtload(lclptr);
 	}
 	settzname();
+out:
+	tzset_guard.release ();
 }
 
 /*
@@ -1771,7 +1792,8 @@ ctime_r(const time_t *timep, char *buf)
 ** Simplified normalize logic courtesy Paul Eggert (eggert@twinsun.com).
 */
 
-static int
+/* Mark as noinline to prevent a compiler warning. */
+static int __attribute__((noinline))
 increment_overflow(int *number, int delta)
 {
 	int	number0;
@@ -1967,7 +1989,28 @@ time2(struct tm *tmp, void (*funcp) P((const time_t*, long, struct tm*)),
 	** If that fails, try with normalization of seconds.
 	*/
 	t = time2sub(tmp, funcp, offset, okayp, false);
-	return *okayp ? t : time2sub(tmp, funcp, offset, okayp, true);
+	if (*okayp)
+	  return t;
+	t = time2sub(tmp, funcp, offset, okayp, true);
+	if (*okayp)
+	  return t;
+	/* Workaround for the spring forward gap problem which results in
+	   the autoconf mktime usability test failing.
+	   What we do here is this:  The gap has 3600 seconds.  If we
+	   subtract 3600 from the tm_sec value and get a valid result,
+	   then we can simply add 3600 to the return value and are done.
+	   If the result is still not valid, the problem is not the
+	   spring forward gap and we can give up. */
+	struct tm tmp2 = *tmp;
+	tmp2.tm_sec -= 3600;
+	t = time2sub(&tmp2, funcp, offset, okayp, true);
+	if (*okayp)
+	  {
+	    if (t + 3600 < 0)	/* Sanity check */
+	      return WRONG;
+	    return t + 3600;
+	  }
+	return t;
 }
 
 static time_t
