@@ -1,6 +1,6 @@
 /* netdb.cc: network database related routines.
 
-   Copyright 2002 Red Hat, Inc.
+   Copyright 2002, 2003, 2007 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -9,10 +9,7 @@ Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
 #include "winsup.h"
-#include <windows.h>
-#include <sys/cygwin.h>
 #include <stdio.h>
-#include <string.h>
 #include <stdlib.h>
 #include <netdb.h>
 
@@ -23,31 +20,19 @@ details. */
  "networks"
  "hosts"
 
- This routine will try to locate these files based on system type.
- Currently the only distinction made is between NT and non-NT systems.
-
  It is the callers responsibility to close the file.  */
 static FILE *
 open_system_file (const char *relative_path)
 {
-  char win32_name[CYG_MAX_PATH];
-  char posix_name[CYG_MAX_PATH];
-  if (wincap.is_winnt ())
-    {
-      if (!GetSystemDirectory (win32_name, CYG_MAX_PATH))
-	return NULL;
-      strcat (win32_name, "\\drivers\\etc\\");
-    }
-  else
-    {
-      if (!GetWindowsDirectory (win32_name, CYG_MAX_PATH))
-	return NULL;
-      strcat (win32_name, "\\");
-    }
+  /* system dir path is never longer. */
+  char win32_name[MAX_PATH];
+
+  if (!GetSystemDirectory (win32_name, MAX_PATH))
+    return NULL;
+  strcat (win32_name, "\\drivers\\etc\\");
   strcat (win32_name, relative_path);
-  cygwin_conv_to_full_posix_path (win32_name, posix_name);
   debug_printf ("netdb file to open %s", win32_name);
-  FILE *result = fopen (posix_name, "rt");
+  FILE *result = fopen (win32_name, "rt");
   debug_printf ("handle to netdb file %p", result);
   return result;
 }
