@@ -1,6 +1,7 @@
 /* registry.cc: registry interface
 
-   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+   2005, 2006, 2007, 2008, 2009 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -9,14 +10,16 @@ Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
 #include "winsup.h"
-#include "shared_info.h"
 #include "registry.h"
-#include "security.h"
-#include <cygwin/version.h>
+#include "cygerrno.h"
+#include "path.h"
+#include "fhandler.h"
+#include "dtable.h"
+#include "cygheap.h"
+#include "tls_pbuf.h"
+#include <wchar.h>
 
-static char NO_COPY cygnus_class[] = "cygnus";
-
-reg_key::reg_key (HKEY top, REGSAM access, ...)
+reg_key::reg_key (HKEY top, REGSAM access, ...): _disposition (0)
 {
   va_list av;
   va_start (av, access);
@@ -24,30 +27,45 @@ reg_key::reg_key (HKEY top, REGSAM access, ...)
   va_end (av);
 }
 
-reg_key::reg_key (REGSAM access, ...)
+/* Opens a key under the appropriate Cygwin key.
+   Do not use HKCU per MS KB 199190  */
+
+reg_key::reg_key (bool isHKLM, REGSAM access, ...): _disposition (0)
 {
   va_list av;
+  HKEY top;
 
-  new (this) reg_key (HKEY_CURRENT_USER, access, "SOFTWARE",
-		 CYGWIN_INFO_CYGNUS_REGISTRY_NAME,
-		 CYGWIN_INFO_CYGWIN_REGISTRY_NAME, NULL);
+  if (isHKLM)
+    top = HKEY_LOCAL_MACHINE;
+  else
+    {
+      char name[128];
+      const char *names[2] = {cygheap->user.get_windows_id (name), ".DEFAULT"};
+      for (int i = 0; i < 2; i++)
+	{
+	  key_is_invalid = RegOpenKeyEx (HKEY_USERS, names[i], 0, access, &top);
+	  if (key_is_invalid == ERROR_SUCCESS)
+	    goto OK;
+	  debug_printf ("HKU\\%s failed, Win32 error %ld", names[i], key_is_invalid);
+	}
+      return;
+    }
+OK:
+  new (this) reg_key (top, access, "SOFTWARE",
+		      CYGWIN_INFO_CYGWIN_REGISTRY_NAME, NULL);
+  if (top != HKEY_LOCAL_MACHINE)
+    RegCloseKey (top);
+  if (key_is_invalid)
+    return;
 
-  HKEY top = key;
+  top = key;
   va_start (av, access);
-  build_reg (top, KEY_READ, av);
+  build_reg (top, access, av);
   va_end (av);
   if (top != key)
     RegCloseKey (top);
 }
 
-reg_key::reg_key (REGSAM access)
-{
-  new (this) reg_key (HKEY_CURRENT_USER, access, "SOFTWARE",
-		 CYGWIN_INFO_CYGNUS_REGISTRY_NAME,
-		 CYGWIN_INFO_CYGWIN_REGISTRY_NAME,
-		 CYGWIN_INFO_CYGWIN_MOUNT_REGISTRY_NAME, NULL);
-}
-
 void
 reg_key::build_reg (HKEY top, REGSAM access, va_list av)
 {
@@ -62,16 +80,15 @@ reg_key::build_reg (HKEY top, REGSAM access, va_list av)
 
   while ((name = va_arg (av, char *)) != NULL)
     {
-      DWORD disp;
       int res = RegCreateKeyExA (r,
 				 name,
 				 0,
-				 cygnus_class,
+				 NULL,
 				 REG_OPTION_NON_VOLATILE,
 				 access,
 				 &sec_none_nih,
 				 &key,
-				 &disp);
+				 &_disposition);
       if (r != top)
 	RegCloseKey (r);
       r = key;
@@ -81,12 +98,6 @@ reg_key::build_reg (HKEY top, REGSAM access, va_list av)
 	  debug_printf ("failed to create key %s in the registry", name);
 	  break;
 	}
-
-      /* If we're considering the mounts key, check if it had to
-	 be created and set had_to_create appropriately. */
-      if (strcmp (name, CYGWIN_INFO_CYGWIN_MOUNT_REGISTRY_NAME) == 0)
-	if (disp == REG_CREATED_NEW_KEY)
-	  mount_table->had_to_create_mount_areas++;
     }
 }
 
@@ -103,8 +114,25 @@ reg_key::get_int (const char *name, int def)
   if (key_is_invalid)
     return def;
 
-  LONG res = RegQueryValueExA (key, name, 0, &type, (unsigned char *) &dst,
-			       &size);
+  LONG res = RegQueryValueExA (key, name, 0, &type, (LPBYTE) &dst, &size);
+
+  if (type != REG_DWORD || res != ERROR_SUCCESS)
+    return def;
+
+  return dst;
+}
+
+int
+reg_key::get_int (const PWCHAR name, int def)
+{
+  DWORD type;
+  DWORD dst;
+  DWORD size = sizeof (dst);
+
+  if (key_is_invalid)
+    return def;
+
+  LONG res = RegQueryValueExW (key, name, 0, &type, (LPBYTE) &dst, &size);
 
   if (type != REG_DWORD || res != ERROR_SUCCESS)
     return def;
@@ -122,14 +150,25 @@ reg_key::set_int (const char *name, int val)
     return key_is_invalid;
 
   return (int) RegSetValueExA (key, name, 0, REG_DWORD,
-			       (unsigned char *) &value, sizeof (value));
+			       (const BYTE *) &value, sizeof (value));
+}
+
+int
+reg_key::set_int (const PWCHAR name, int val)
+{
+  DWORD value = val;
+  if (key_is_invalid)
+    return key_is_invalid;
+
+  return (int) RegSetValueExW (key, name, 0, REG_DWORD,
+			       (const BYTE *) &value, sizeof (value));
 }
 
 /* Given the current registry key, return the specific string value
    requested.  Return zero on success, non-zero on failure. */
 
 int
-reg_key::get_string (const char *name, char *dst, size_t max, const char * def)
+reg_key::get_string (const char *name, char *dst, size_t max, const char *def)
 {
   DWORD size = max;
   DWORD type;
@@ -138,13 +177,30 @@ reg_key::get_string (const char *name, char *dst, size_t max, const char * def)
   if (key_is_invalid)
     res = key_is_invalid;
   else
-    res = RegQueryValueExA (key, name, 0, &type, (unsigned char *) dst, &size);
+    res = RegQueryValueExA (key, name, 0, &type, (LPBYTE) dst, &size);
 
   if ((def != 0) && ((type != REG_SZ) || (res != ERROR_SUCCESS)))
     strcpy (dst, def);
   return (int) res;
 }
 
+int
+reg_key::get_string (const PWCHAR name, PWCHAR dst, size_t max, const PWCHAR def)
+{
+  DWORD size = max;
+  DWORD type;
+  LONG res;
+
+  if (key_is_invalid)
+    res = key_is_invalid;
+  else
+    res = RegQueryValueExW (key, name, 0, &type, (LPBYTE) dst, &size);
+
+  if ((def != 0) && ((type != REG_SZ) || (res != ERROR_SUCCESS)))
+    wcscpy (dst, def);
+  return (int) res;
+}
+
 /* Given the current registry key, set a specific string value. */
 
 int
@@ -152,10 +208,19 @@ reg_key::set_string (const char *name, const char *src)
 {
   if (key_is_invalid)
     return key_is_invalid;
-  return (int) RegSetValueExA (key, name, 0, REG_SZ, (unsigned char*) src,
+  return (int) RegSetValueExA (key, name, 0, REG_SZ, (const BYTE*) src,
 			       strlen (src) + 1);
 }
 
+int
+reg_key::set_string (const PWCHAR name, const PWCHAR src)
+{
+  if (key_is_invalid)
+    return key_is_invalid;
+  return (int) RegSetValueExW (key, name, 0, REG_SZ, (const BYTE*) src,
+			       (wcslen (src) + 1) * sizeof (WCHAR));
+}
+
 /* Return the handle to key. */
 
 HKEY
@@ -193,62 +258,56 @@ reg_key::~reg_key ()
   key_is_invalid = 1;
 }
 
-char *
-get_registry_hive_path (const PSID psid, char *path)
+PWCHAR
+get_registry_hive_path (const PWCHAR name, PWCHAR path)
 {
-  char sid[256];
-  char key[256];
+  WCHAR key[256], *kend;
   HKEY hkey;
 
-  if (!psid || !path)
+  if (!name || !path)
     return NULL;
-  cygpsid csid (psid);
-  csid.string (sid);
-  strcpy (key,"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList\\");
-  strcat (key, sid);
-  if (!RegOpenKeyExA (HKEY_LOCAL_MACHINE, key, 0, KEY_READ, &hkey))
+  kend = wcpcpy (key, L"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList\\");
+  wcpcpy (kend, name);
+  if (!RegOpenKeyExW (HKEY_LOCAL_MACHINE, key, 0, KEY_READ, &hkey))
     {
-      char buf[256];
+      tmp_pathbuf tp;
+      PWCHAR buf = tp.w_get ();
       DWORD type, siz;
 
-      path[0] = '\0';
-      if (!RegQueryValueExA (hkey, "ProfileImagePath", 0, &type,
-			     (BYTE *)buf, (siz = sizeof (buf), &siz)))
-	ExpandEnvironmentStringsA (buf, path, CYG_MAX_PATH + 1);
+      path[0] = L'\0';
+      if (!RegQueryValueExW (hkey, L"ProfileImagePath", 0, &type,
+			     (BYTE *)buf, (siz = NT_MAX_PATH, &siz)))
+	ExpandEnvironmentStringsW (buf, path, NT_MAX_PATH);
       RegCloseKey (hkey);
       if (path[0])
 	return path;
     }
-  debug_printf ("HKLM\\%s not found", key);
+  debug_printf ("HKLM\\%W not found", key);
   return NULL;
 }
 
 void
-load_registry_hive (PSID psid)
+load_registry_hive (const PWCHAR name)
 {
-  char sid[256];
-  char path[CYG_MAX_PATH + 1];
+  tmp_pathbuf tp;
+  PWCHAR path = tp.w_get ();
   HKEY hkey;
   LONG ret;
 
-  if (!psid)
+  if (!name)
     return;
   /* Check if user hive is already loaded. */
-  cygpsid csid (psid);
-  csid.string (sid);
-  if (!RegOpenKeyExA (HKEY_USERS, sid, 0, KEY_READ, &hkey))
+  if (!RegOpenKeyExW (HKEY_USERS, name, 0, KEY_READ, &hkey))
     {
-      debug_printf ("User registry hive for %s already exists", sid);
+      debug_printf ("User registry hive for %W already exists", name);
       RegCloseKey (hkey);
       return;
     }
-  /* This is only called while deimpersonated */
-  set_process_privilege (SE_RESTORE_NAME);
-  if (get_registry_hive_path (psid, path))
+  if (get_registry_hive_path (name, path))
     {
-      strcat (path, "\\NTUSER.DAT");
-      if ((ret = RegLoadKeyA (HKEY_USERS, sid, path)) != ERROR_SUCCESS)
-	debug_printf ("Loading user registry hive for %s failed: %d", sid, ret);
+      wcscat (path, L"\\NTUSER.DAT");
+      if ((ret = RegLoadKeyW (HKEY_USERS, name, path)) != ERROR_SUCCESS)
+	debug_printf ("Loading user registry hive for %W failed: %d", name, ret);
     }
 }
 
