@@ -1,6 +1,7 @@
 /* sec_acl.cc: Sun compatible ACL functions.
 
-   Copyright 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
+   Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+   2009, 2010 Red Hat, Inc.
 
    Written by Corinna Vinschen <corinna@vinschen.de>
 
@@ -11,29 +12,17 @@ Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
 #include "winsup.h"
-#include <grp.h>
-#include <pwd.h>
-#include <unistd.h>
 #include <stdlib.h>
-#include <limits.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/acl.h>
 #include <ctype.h>
-#include <wingdi.h>
-#include <winuser.h>
 #include "cygerrno.h"
 #include "security.h"
 #include "path.h"
 #include "fhandler.h"
 #include "dtable.h"
-#include "pinfo.h"
 #include "cygheap.h"
 #include "pwdgrp.h"
 
-extern "C" int aclsort32 (int nentries, int, __aclent32_t *aclbufp);
-extern "C" int acl32 (const char *path, int cmd, int nentries, __aclent32_t *aclbufp);
-
 static int
 searchace (__aclent32_t *aclp, int nentries, int type, __uid32_t id = ILLEGAL_UID)
 {
@@ -46,16 +35,14 @@ searchace (__aclent32_t *aclp, int nentries, int type, __uid32_t id = ILLEGAL_UI
   return -1;
 }
 
-static int
-setacl (const char *file, int nentries, __aclent32_t *aclbufp)
+int
+setacl (HANDLE handle, path_conv &pc, int nentries, __aclent32_t *aclbufp,
+	bool &writable)
 {
   security_descriptor sd_ret;
 
-  if (read_sd (file, sd_ret) <= 0)
-    {
-      debug_printf ("read_sd %E");
-      return -1;
-    }
+  if (get_file_sd (handle, pc, sd_ret))
+    return -1;
 
   BOOL dummy;
 
@@ -96,8 +83,7 @@ setacl (const char *file, int nentries, __aclent32_t *aclbufp)
     }
 
   /* Fill access control list. */
-  char acl_buf[3072];
-  PACL acl = (PACL) acl_buf;
+  PACL acl = (PACL) alloca (3072);
   size_t acl_len = sizeof (ACL);
   int ace_off = 0;
 
@@ -111,25 +97,35 @@ setacl (const char *file, int nentries, __aclent32_t *aclbufp)
       __seterrno ();
       return -1;
     }
+
+  writable = false;
+
   for (int i = 0; i < nentries; ++i)
     {
       DWORD allow;
       /* Owner has more standard rights set. */
       if ((aclbufp[i].a_type & ~ACL_DEFAULT) == USER_OBJ)
-	allow = STANDARD_RIGHTS_ALL | FILE_WRITE_ATTRIBUTES | FILE_WRITE_EA;
+	allow = STANDARD_RIGHTS_ALL
+		| (pc.fs_is_samba ()
+		   ? 0 : (FILE_READ_ATTRIBUTES | FILE_WRITE_ATTRIBUTES));
       else
-	allow = STANDARD_RIGHTS_READ | FILE_READ_ATTRIBUTES | FILE_READ_EA;
+	allow = STANDARD_RIGHTS_READ
+		| (pc.fs_is_samba () ? 0 : FILE_READ_ATTRIBUTES);
       if (aclbufp[i].a_perm & S_IROTH)
 	allow |= FILE_GENERIC_READ;
       if (aclbufp[i].a_perm & S_IWOTH)
-	allow |= STANDARD_RIGHTS_WRITE | FILE_GENERIC_WRITE;
+	{
+	  allow |= FILE_GENERIC_WRITE;
+	  writable = true;
+	}
       if (aclbufp[i].a_perm & S_IXOTH)
-	allow |= FILE_GENERIC_EXECUTE;
+	allow |= FILE_GENERIC_EXECUTE & ~FILE_READ_ATTRIBUTES;
       if ((aclbufp[i].a_perm & (S_IWOTH | S_IXOTH)) == (S_IWOTH | S_IXOTH))
 	allow |= FILE_DELETE_CHILD;
       /* Set inherit property. */
       DWORD inheritance = (aclbufp[i].a_type & ACL_DEFAULT)
-			  ? (SUB_CONTAINERS_AND_OBJECTS_INHERIT | INHERIT_ONLY)
+			  ? (CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE
+			     | INHERIT_ONLY_ACE)
 			  : NO_INHERITANCE;
       /*
        * If a specific acl contains a corresponding default entry with
@@ -142,11 +138,11 @@ setacl (const char *file, int nentries, __aclent32_t *aclbufp)
 			       aclbufp[i].a_type | ACL_DEFAULT,
 			       (aclbufp[i].a_type & (USER|GROUP))
 			       ? aclbufp[i].a_id : ILLEGAL_UID)) >= 0
-	  && aclbufp[i].a_perm == aclbufp[pos].a_perm)
+	  && aclbufp[i].a_perm == aclbufp[i + 1 + pos].a_perm)
 	{
-	  inheritance = SUB_CONTAINERS_AND_OBJECTS_INHERIT;
+	  inheritance = CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE;
 	  /* This invalidates the corresponding default entry. */
-	  aclbufp[pos].a_type = USER|GROUP|ACL_DEFAULT;
+	  aclbufp[i + 1 + pos].a_type = USER|GROUP|ACL_DEFAULT;
 	}
       switch (aclbufp[i].a_type)
 	{
@@ -163,9 +159,13 @@ setacl (const char *file, int nentries, __aclent32_t *aclbufp)
 	case USER:
 	case DEF_USER:
 	  if (!(pw = internal_getpwuid (aclbufp[i].a_id))
-	      || !sid.getfrompw (pw)
-	      || !add_access_allowed_ace (acl, ace_off++, allow,
-					  sid, acl_len, inheritance))
+	      || !sid.getfrompw (pw))
+	    {
+	      set_errno (EINVAL);
+	      return -1;
+	    }
+	  if (!add_access_allowed_ace (acl, ace_off++, allow,
+				       sid, acl_len, inheritance))
 	    return -1;
 	  break;
 	case GROUP_OBJ:
@@ -181,9 +181,13 @@ setacl (const char *file, int nentries, __aclent32_t *aclbufp)
 	case GROUP:
 	case DEF_GROUP:
 	  if (!(gr = internal_getgrgid (aclbufp[i].a_id))
-	      || !sid.getfromgr (gr)
-	      || !add_access_allowed_ace (acl, ace_off++, allow,
-					  sid, acl_len, inheritance))
+	      || !sid.getfromgr (gr))
+	    {
+	      set_errno (EINVAL);
+	      return -1;
+	    }
+	  if (!add_access_allowed_ace (acl, ace_off++, allow,
+				       sid, acl_len, inheritance))
 	    return -1;
 	  break;
 	case OTHER_OBJ:
@@ -223,7 +227,7 @@ setacl (const char *file, int nentries, __aclent32_t *aclbufp)
       return -1;
     }
   debug_printf ("Created SD-Size: %d", sd_ret.size ());
-  return write_sd (file, sd_ret);
+  return set_file_sd (handle, pc, sd_ret, false);
 }
 
 /* Temporary access denied bits */
@@ -239,35 +243,37 @@ getace (__aclent32_t &acl, int type, int id, DWORD win_ace_mask,
   acl.a_id = id;
 
   if ((win_ace_mask & FILE_READ_BITS) && !(acl.a_perm & (S_IROTH | DENY_R)))
-    if (win_ace_type == ACCESS_ALLOWED_ACE_TYPE)
-      acl.a_perm |= S_IROTH;
-    else if (win_ace_type == ACCESS_DENIED_ACE_TYPE)
-      acl.a_perm |= DENY_R;
+    {
+      if (win_ace_type == ACCESS_ALLOWED_ACE_TYPE)
+	acl.a_perm |= S_IROTH;
+      else if (win_ace_type == ACCESS_DENIED_ACE_TYPE)
+	acl.a_perm |= DENY_R;
+    }
 
   if ((win_ace_mask & FILE_WRITE_BITS) && !(acl.a_perm & (S_IWOTH | DENY_W)))
-    if (win_ace_type == ACCESS_ALLOWED_ACE_TYPE)
-      acl.a_perm |= S_IWOTH;
-    else if (win_ace_type == ACCESS_DENIED_ACE_TYPE)
-      acl.a_perm |= DENY_W;
+    {
+      if (win_ace_type == ACCESS_ALLOWED_ACE_TYPE)
+	acl.a_perm |= S_IWOTH;
+      else if (win_ace_type == ACCESS_DENIED_ACE_TYPE)
+	acl.a_perm |= DENY_W;
+    }
 
   if ((win_ace_mask & FILE_EXEC_BITS) && !(acl.a_perm & (S_IXOTH | DENY_X)))
-    if (win_ace_type == ACCESS_ALLOWED_ACE_TYPE)
-      acl.a_perm |= S_IXOTH;
-    else if (win_ace_type == ACCESS_DENIED_ACE_TYPE)
-      acl.a_perm |= DENY_X;
+    {
+      if (win_ace_type == ACCESS_ALLOWED_ACE_TYPE)
+	acl.a_perm |= S_IXOTH;
+      else if (win_ace_type == ACCESS_DENIED_ACE_TYPE)
+	acl.a_perm |= DENY_X;
+    }
 }
 
-static int
-getacl (const char *file, DWORD attr, int nentries, __aclent32_t *aclbufp)
+int
+getacl (HANDLE handle, path_conv &pc, int nentries, __aclent32_t *aclbufp)
 {
   security_descriptor sd;
 
-  int ret;
-  if ((ret = read_sd (file, sd)) <= 0)
-    {
-      debug_printf ("read_sd %E");
-      return ret;
-    }
+  if (get_file_sd (handle, pc, sd))
+    return -1;
 
   cygpsid owner_sid;
   cygpsid group_sid;
@@ -361,13 +367,14 @@ getacl (const char *file, DWORD attr, int nentries, __aclent32_t *aclbufp)
 
 	  if (!type)
 	    continue;
-	  if (!(ace->Header.AceFlags & INHERIT_ONLY || type & ACL_DEFAULT))
+	  if (!(ace->Header.AceFlags & INHERIT_ONLY_ACE || type & ACL_DEFAULT))
 	    {
 	      if ((pos = searchace (lacl, MAX_ACL_ENTRIES, type, id)) >= 0)
 		getace (lacl[pos], type, id, ace->Mask, ace->Header.AceType);
 	    }
-	  if ((ace->Header.AceFlags & SUB_CONTAINERS_AND_OBJECTS_INHERIT)
-	      && (attr & FILE_ATTRIBUTE_DIRECTORY))
+	  if ((ace->Header.AceFlags
+	      & (CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE))
+	      && pc.isdir ())
 	    {
 	      if (type == USER_OBJ)
 		type = USER;
@@ -403,99 +410,41 @@ getacl (const char *file, DWORD attr, int nentries, __aclent32_t *aclbufp)
       aclbufp[i].a_perm &= ~(DENY_R | DENY_W | DENY_X);
     aclsort32 (pos, 0, aclbufp);
   }
-  syscall_printf ("%d = getacl (%s)", pos, file);
+  syscall_printf ("%d = getacl (%S)", pos, pc.get_nt_native_path ());
   return pos;
 }
 
 static int
 acl_worker (const char *path, int cmd, int nentries, __aclent32_t *aclbufp,
-	    int nofollow)
+	    unsigned fmode)
 {
-  extern suffix_info stat_suffixes[];
-  path_conv real_path (path, (nofollow ? PC_SYM_NOFOLLOW : PC_SYM_FOLLOW) | PC_FULL, stat_suffixes);
-  if (real_path.error)
+  int res = -1;
+  fhandler_base *fh = build_fh_name (path, fmode, stat_suffixes);
+  if (fh->error ())
     {
-      set_errno (real_path.error);
-      syscall_printf ("-1 = acl (%s)", path);
-      return -1;
+      debug_printf ("got %d error from build_fh_name", fh->error ());
+      set_errno (fh->error ());
     }
-  if (!real_path.has_acls () || !allow_ntsec)
-    {
-      struct __stat64 st;
-      int ret = -1;
+  else if (!fh->exists ())
+    set_errno (ENOENT);
+  else
+    res = fh->facl (cmd, nentries, aclbufp);
 
-      switch (cmd)
-	{
-	case SETACL:
-	  set_errno (ENOSYS);
-	  break;
-	case GETACL:
-	  if (!aclbufp)
-	    set_errno(EFAULT);
-	  else if (nentries < MIN_ACL_ENTRIES)
-	    set_errno (ENOSPC);
-	  else if ((nofollow && !lstat64 (path, &st))
-		   || (!nofollow && !stat64 (path, &st)))
-	    {
-	      aclbufp[0].a_type = USER_OBJ;
-	      aclbufp[0].a_id = st.st_uid;
-	      aclbufp[0].a_perm = (st.st_mode & S_IRWXU) >> 6;
-	      aclbufp[1].a_type = GROUP_OBJ;
-	      aclbufp[1].a_id = st.st_gid;
-	      aclbufp[1].a_perm = (st.st_mode & S_IRWXG) >> 3;
-	      aclbufp[2].a_type = OTHER_OBJ;
-	      aclbufp[2].a_id = ILLEGAL_GID;
-	      aclbufp[2].a_perm = st.st_mode & S_IRWXO;
-	      aclbufp[3].a_type = CLASS_OBJ;
-	      aclbufp[3].a_id = ILLEGAL_GID;
-	      aclbufp[3].a_perm = S_IRWXU | S_IRWXG | S_IRWXO;
-	      ret = MIN_ACL_ENTRIES;
-	    }
-	  break;
-	case GETACLCNT:
-	  ret = MIN_ACL_ENTRIES;
-	  break;
-	}
-      syscall_printf ("%d = acl (%s)", ret, path);
-      return ret;
-    }
-  switch (cmd)
-    {
-      case SETACL:
-	if (!aclsort32 (nentries, 0, aclbufp))
-	  return setacl (real_path.get_win32 (),
-			 nentries, aclbufp);
-	break;
-      case GETACL:
-	if (!aclbufp)
-	  set_errno(EFAULT);
-	else
-	  return getacl (real_path.get_win32 (),
-			 real_path.file_attributes (),
-			 nentries, aclbufp);
-	break;
-      case GETACLCNT:
-	return getacl (real_path.get_win32 (),
-		       real_path.file_attributes (),
-		       0, NULL);
-      default:
-	set_errno (EINVAL);
-	break;
-    }
-  syscall_printf ("-1 = acl (%s)", path);
-  return -1;
+  delete fh;
+  syscall_printf ("%d = acl (%s)", res, path);
+  return res;
 }
 
 extern "C" int
 acl32 (const char *path, int cmd, int nentries, __aclent32_t *aclbufp)
 {
-  return acl_worker (path, cmd, nentries, aclbufp, 0);
+  return acl_worker (path, cmd, nentries, aclbufp, PC_SYM_FOLLOW);
 }
 
 extern "C" int
 lacl32 (const char *path, int cmd, int nentries, __aclent32_t *aclbufp)
 {
-  return acl_worker (path, cmd, nentries, aclbufp, 1);
+  return acl_worker (path, cmd, nentries, aclbufp, PC_SYM_NOFOLLOW);
 }
 
 extern "C" int
@@ -507,15 +456,9 @@ facl32 (int fd, int cmd, int nentries, __aclent32_t *aclbufp)
       syscall_printf ("-1 = facl (%d)", fd);
       return -1;
     }
-  const char *path = cfd->get_name ();
-  if (path == NULL)
-    {
-      syscall_printf ("-1 = facl (%d) (no name)", fd);
-      set_errno (ENOSYS);
-      return -1;
-    }
-  syscall_printf ("facl (%d): calling acl (%s)", fd, path);
-  return acl_worker (path, cmd, nentries, aclbufp, 0);
+  int res = cfd->facl (cmd, nentries, aclbufp);
+  syscall_printf ("%d = facl (%s) )", res, cfd->get_name ());
+  return res;
 }
 
 extern "C" int
@@ -795,7 +738,7 @@ acltotext32 (__aclent32_t *aclbufp, int aclcnt)
       first = false;
       if (aclbufp[pos].a_type & ACL_DEFAULT)
 	strcat (buf, "default");
-      switch (aclbufp[pos].a_type)
+      switch (aclbufp[pos].a_type & ~ACL_DEFAULT)
 	{
 	case USER_OBJ:
 	  __small_sprintf (buf + strlen (buf), "user::%s",
