@@ -1,6 +1,7 @@
 /* tty.cc
 
-   Copyright 1997, 1998, 2000, 2001, 2002 Red Hat, Inc.
+   Copyright 1997, 1998, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008, 2009
+   Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -9,10 +10,9 @@ Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
 details. */
 
 #include "winsup.h"
+#include "miscfuncs.h"
 #include <unistd.h>
 #include <utmp.h>
-#include <wingdi.h>
-#include <winuser.h>
 #include <sys/cygwin.h>
 #include "cygerrno.h"
 #include "security.h"
@@ -21,12 +21,16 @@ details. */
 #include "dtable.h"
 #include "cygheap.h"
 #include "pinfo.h"
-#include "cygserver.h"
 #include "shared_info.h"
-#include "cygthread.h"
 
 extern fhandler_tty_master *tty_master;
 
+extern "C" int
+posix_openpt (int oflags)
+{
+  return open ("/dev/ptmx", oflags);
+}
+
 extern "C" int
 grantpt (int fd)
 {
@@ -54,19 +58,35 @@ ttyslot (void)
   return myself->ctty;
 }
 
+HANDLE NO_COPY tty_list::mutex = NULL;
+
 void __stdcall
-tty_init (void)
+tty_list::init_session ()
 {
-  if (!myself->ppid_handle && NOTSTATE (myself, PID_CYGPARENT))
+  char mutex_name[MAX_PATH];
+  /* tty_list::mutex is used while searching for a tty slot. It's necessary
+     while finding console window handle */
+
+  char *name = shared_name (mutex_name, "tty_list::mutex", 0);
+  if (!(mutex = CreateMutex (&sec_all_nih, FALSE, name)))
+    api_fatal ("can't create tty_list::mutex '%s', %E", name);
+  ProtectHandle (mutex);
+}
+
+void __stdcall
+tty::init_session ()
+{
+  if (!myself->cygstarted && NOTSTATE (myself, PID_CYGPARENT))
     cygheap->fdtab.get_debugger_info ();
 
   if (NOTSTATE (myself, PID_USETTY))
     return;
-  if (myself->ctty == -1)
-    if (NOTSTATE (myself, PID_CYGPARENT))
-      myself->ctty = attach_tty (myself->ctty);
-    else
-      return;
+  if (myself->ctty != -1)
+    /* nothing to do */;
+  else if (NOTSTATE (myself, PID_CYGPARENT))
+    myself->ctty = cygwin_shared->tty.attach (myself->ctty);
+  else
+    return;
   if (myself->ctty == -1)
     termios_printf ("Can't attach to tty");
 }
@@ -74,13 +94,13 @@ tty_init (void)
 /* Create session's master tty */
 
 void __stdcall
-create_tty_master (int ttynum)
+tty::create_master (int ttynum)
 {
   device ttym = *ttym_dev;
   ttym.setunit (ttynum); /* CGF FIXME device */
   tty_master = (fhandler_tty_master *) build_fh_dev (ttym);
   if (tty_master->init ())
-    api_fatal ("Can't create master tty");
+    api_fatal ("can't create master tty");
   else
     {
       /* Log utmp entry */
@@ -88,7 +108,7 @@ create_tty_master (int ttynum)
       DWORD len = sizeof our_utmp.ut_host;
 
       bzero ((char *) &our_utmp, sizeof (utmp));
-      (void) time (&our_utmp.ut_time);
+      time (&our_utmp.ut_time);
       strncpy (our_utmp.ut_name, getlogin (), sizeof (our_utmp.ut_name));
       GetComputerName (our_utmp.ut_host, &len);
       __small_sprintf (our_utmp.ut_line, "tty%d", ttynum);
@@ -104,37 +124,29 @@ create_tty_master (int ttynum)
     }
 }
 
-void __stdcall
-tty_terminate (void)
-{
-  if (NOTSTATE (myself, PID_USETTY))
-    return;
-  cygwin_shared->tty.terminate ();
-}
-
 int __stdcall
-attach_tty (int num)
+tty_list::attach (int num)
 {
   if (num != -1)
     {
-      return cygwin_shared->tty.connect_tty (num);
+      return connect (num);
     }
   if (NOTSTATE (myself, PID_USETTY))
     return -1;
-  return cygwin_shared->tty.allocate_tty (true);
+  return allocate (true);
 }
 
 void
-tty_list::terminate (void)
+tty_list::terminate ()
 {
+  if (NOTSTATE (myself, PID_USETTY))
+    return;
   int ttynum = myself->ctty;
 
   /* Keep master running till there are connected clients */
-  if (ttynum != -1 && ttys[ttynum].master_pid == GetCurrentProcessId ())
+  if (ttynum != -1 && tty_master && ttys[ttynum].master_pid == myself->pid)
     {
       tty *t = ttys + ttynum;
-      CloseHandle (t->from_master);
-      CloseHandle (t->to_master);
       /* Wait for children which rely on tty handling in this process to
 	 go away */
       for (int i = 0; ; i++)
@@ -148,13 +160,17 @@ tty_list::terminate (void)
 	      i = 0;
 	    }
 
-	  low_priority_sleep (200);
+	  Sleep (200);
 	}
 
+      lock_ttys here ();
+      CloseHandle (tty_master->from_master);
+      CloseHandle (tty_master->to_master);
+
       termios_printf ("tty %d master about to finish", ttynum);
-      ForceCloseHandle1 (t->to_slave, to_pty);
-      ForceCloseHandle1 (t->from_slave, from_pty);
-      CloseHandle (tty_master->inuse);
+      CloseHandle (tty_master->get_io_handle ());
+      CloseHandle (tty_master->get_output_handle ());
+
       t->init ();
 
       char buf[20];
@@ -164,7 +180,7 @@ tty_list::terminate (void)
 }
 
 int
-tty_list::connect_tty (int ttynum)
+tty_list::connect (int ttynum)
 {
   if (ttynum < 0 || ttynum >= NTTYS)
     {
@@ -181,7 +197,7 @@ tty_list::connect_tty (int ttynum)
 }
 
 void
-tty_list::init (void)
+tty_list::init ()
 {
   for (int i = 0; i < NTTYS; i++)
     {
@@ -193,49 +209,23 @@ tty_list::init (void)
 /* Search for tty class for our console. Allocate new tty if our process is
    the only cygwin process in the current console.
    Return tty number or -1 if error.
-   If flag == 0, just find a free tty.
+   If with_console == 0, just find a free tty.
  */
 int
-tty_list::allocate_tty (bool with_console)
+tty_list::allocate (bool with_console)
 {
   HWND console;
   int freetty = -1;
   HANDLE hmaster = NULL;
 
-  /* FIXME: This whole function needs a protective mutex. */
-
-  if (WaitForSingleObject (tty_mutex, INFINITE) == WAIT_FAILED)
-    termios_printf ("WFSO for tty_mutex %p failed, %E", tty_mutex);
+  lock_ttys here;
 
   if (!with_console)
     console = NULL;
   else if (!(console = GetConsoleWindow ()))
     {
-      char oldtitle[TITLESIZE];
-
-      if (!GetConsoleTitle (oldtitle, TITLESIZE))
-	{
-	  termios_printf ("Can't read console title");
-	  goto out;
-	}
-
-      char buf[40];
-
-      __small_sprintf (buf, "cygwin.find.console.%d", myself->pid);
-      SetConsoleTitle (buf);
-      for (int times = 0; times < 25; times++)
-	{
-	  Sleep (10);
-	  if ((console = FindWindow (NULL, buf)))
-	    break;
-	}
-      SetConsoleTitle (oldtitle);
-      Sleep (40);
-      if (console == NULL)
-	{
-	  termios_printf ("Can't find console window");
-	  goto out;
-	}
+      termios_printf ("Can't find console window");
+      goto out;
     }
 
   /* Is a tty allocated for console? */
@@ -269,7 +259,7 @@ tty_list::allocate_tty (bool with_console)
 	}
     }
 
-  /* There is no tty allocated to console, allocate the first free found */
+  /* There is no tty allocated to console; allocate the first free found */
   if (freetty == -1)
     goto out;
 
@@ -277,26 +267,21 @@ tty_list::allocate_tty (bool with_console)
   t = ttys + freetty;
   t->init ();
   t->setsid (-1);
-  t->setpgid (myself->pgid);
   t->sethwnd (console);
 
 out:
   if (freetty < 0)
-    {
-      ReleaseMutex (tty_mutex);
-      system_printf ("No tty allocated");
-    }
+    system_printf ("No tty allocated");
   else if (!with_console)
     {
       termios_printf ("tty%d allocated", freetty);
-      /* exit with tty_mutex still held -- caller has more work to do */
+      here.dont_release (); /* exit with mutex still held -- caller has more work to do */
     }
   else
     {
       termios_printf ("console %p associated with tty%d", console, freetty);
       if (!hmaster)
-	create_tty_master (freetty);
-      ReleaseMutex (tty_mutex);
+	tty::create_master (freetty);
     }
   return freetty;
 }
@@ -308,16 +293,34 @@ tty::slave_alive ()
 }
 
 bool
-tty::master_alive ()
+tty::exists ()
 {
-  return alive (TTY_MASTER_ALIVE);
+  /* Attempt to open the from-master side of the tty.  If it is accessible
+     then it exists although it may have been privileges to actually use it. */
+  char pipename[sizeof("ttyNNNN-from-master")];
+  __small_sprintf (pipename, "tty%d-from-master", ntty);
+  HANDLE r, w;
+  int res = fhandler_pipe::create_selectable (&sec_none_nih, r, w, 0, pipename);
+  if (res)
+    return true;
+
+  CloseHandle (r);
+  CloseHandle (w);
+
+  HANDLE h = open_output_mutex ();
+  if (h)
+    {
+      CloseHandle (h);
+      return true;
+    }
+  return slave_alive ();
 }
 
 bool
 tty::alive (const char *fmt)
 {
   HANDLE ev;
-  char buf[CYG_MAX_PATH];
+  char buf[MAX_PATH];
 
   shared_name (buf, fmt, ntty);
   if ((ev = OpenEvent (EVENT_ALL_ACCESS, FALSE, buf)))
@@ -340,7 +343,7 @@ tty::open_input_mutex ()
 HANDLE
 tty::open_mutex (const char *mutex)
 {
-  char buf[CYG_MAX_PATH];
+  char buf[MAX_PATH];
   shared_name (buf, mutex, ntty);
   return OpenMutex (MUTEX_ALL_ACCESS, TRUE, buf);
 }
@@ -349,7 +352,7 @@ HANDLE
 tty::create_inuse (const char *fmt)
 {
   HANDLE h;
-  char buf[CYG_MAX_PATH];
+  char buf[MAX_PATH];
 
   shared_name (buf, fmt, ntty);
   h = CreateEvent (&sec_all, TRUE, FALSE, buf);
@@ -360,22 +363,21 @@ tty::create_inuse (const char *fmt)
 }
 
 void
-tty::init (void)
+tty::init ()
 {
   output_stopped = 0;
   setsid (0);
   pgid = 0;
   hwnd = NULL;
-  to_slave = NULL;
-  from_slave = NULL;
   was_opened = 0;
+  master_pid = 0;
 }
 
 HANDLE
 tty::get_event (const char *fmt, BOOL manual_reset)
 {
   HANDLE hev;
-  char buf[CYG_MAX_PATH];
+  char buf[MAX_PATH];
 
   shared_name (buf, fmt, ntty);
   if (!(hev = CreateEvent (&sec_all, manual_reset, FALSE, buf)))
@@ -389,106 +391,17 @@ tty::get_event (const char *fmt, BOOL manual_reset)
   return hev;
 }
 
-bool
-tty::make_pipes (fhandler_pty_master *ptym)
+lock_ttys::lock_ttys (DWORD howlong): release_me (true)
 {
-  /* Create communication pipes */
-
-  /* FIXME: should this be sec_none_nih? */
-  if (!CreatePipe (&from_master, &to_slave, &sec_all, 128 * 1024))
+  if (WaitForSingleObject (tty_list::mutex, howlong) == WAIT_FAILED)
     {
-      termios_printf ("can't create input pipe");
-      set_errno (ENOENT);
-      return false;
+      termios_printf ("WFSO for mutex %p failed, %E", tty_list::mutex);
+      release_me = false;
     }
-
-  // ProtectHandle1INH (to_slave, to_pty);
-  if (!CreatePipe (&from_slave, &to_master, &sec_all, 128 * 1024))
-    {
-      termios_printf ("can't create output pipe");
-      set_errno (ENOENT);
-      return false;
-    }
-  // ProtectHandle1INH (from_slave, from_pty);
-  termios_printf ("tty%d from_slave %p, to_slave %p", ntty, from_slave,
-		  to_slave);
-
-  DWORD pipe_mode = PIPE_NOWAIT;
-  if (!SetNamedPipeHandleState (to_slave, &pipe_mode, NULL, NULL))
-    termios_printf ("can't set to_slave to non-blocking mode");
-  ptym->set_io_handle (from_slave);
-  ptym->set_output_handle (to_slave);
-  return true;
 }
 
-bool
-tty::common_init (fhandler_pty_master *ptym)
+void
+lock_ttys::release ()
 {
-  /* Set termios information.  Force initialization. */
-  ptym->tcinit (this, true);
-
-  if (!make_pipes (ptym))
-    return false;
-  ptym->need_nl = 0;
-
-  /* Save our pid  */
-
-  master_pid = GetCurrentProcessId ();
-
-  /* We do not open allow the others to open us (for handle duplication)
-     but rely on cygheap->inherited_ctty for descendant processes.
-     In the future the cygserver may allow access by others. */
-
-#ifdef USE_SERVER
-  if (wincap.has_security ())
-    {
-      if (cygserver_running == CYGSERVER_UNKNOWN)
-	cygserver_init ();
-    }
-#endif
-
-  /* Create synchronisation events */
-
-  if (ptym->get_major () != DEV_TTYM_MAJOR)
-    {
-      ptym->output_done_event = ptym->ioctl_done_event =
-      ptym->ioctl_request_event = NULL;
-    }
-  else
-    {
-      if (!(ptym->output_done_event = get_event (OUTPUT_DONE_EVENT)))
-	return false;
-      if (!(ptym->ioctl_done_event = get_event (IOCTL_DONE_EVENT)))
-	return false;
-      if (!(ptym->ioctl_request_event = get_event (IOCTL_REQUEST_EVENT)))
-	return false;
-    }
-
-  if (!(ptym->input_available_event = get_event (INPUT_AVAILABLE_EVENT, TRUE)))
-    return false;
-
-  char buf[CYG_MAX_PATH];
-  shared_name (buf, OUTPUT_MUTEX, ntty);
-  if (!(ptym->output_mutex = CreateMutex (&sec_all, FALSE, buf)))
-    {
-      termios_printf ("can't create %s", buf);
-      set_errno (ENOENT);
-      return false;
-    }
-
-  shared_name (buf, INPUT_MUTEX, ntty);
-  if (!(ptym->input_mutex = CreateMutex (&sec_all, FALSE, buf)))
-    {
-      termios_printf ("can't create %s", buf);
-      set_errno (ENOENT);
-      return false;
-    }
-
-  // /* screws up tty master */ ProtectHandle1INH (ptym->output_mutex, output_mutex);
-  // /* screws up tty master */ ProtectHandle1INH (ptym->input_mutex, input_mutex);
-  winsize.ws_col = 80;
-  winsize.ws_row = 25;
-
-  termios_printf ("tty%d opened", ntty);
-  return true;
+  ReleaseMutex (tty_list::mutex);
 }
