@@ -1,6 +1,6 @@
 /* passwd.c: Changing passwords and managing account information
 
-   Copyright 1999, 2000, 2001, 2002, 2003 Red Hat, Inc.
+   Copyright 1999, 2000, 2001, 2002, 2003, 2008, 2009 Red Hat, Inc.
 
    Written by Corinna Vinschen <corinna.vinschen@cityweb.de>
 
@@ -26,6 +26,9 @@ details. */
 #include <sys/cygwin.h>
 #include <sys/types.h>
 #include <time.h>
+#include <errno.h>
+#include <locale.h>
+#include <wchar.h>
 
 #define USER_PRIV_ADMIN		 2
 
@@ -36,6 +39,7 @@ static struct option longopts[] =
 {
   {"cannot-change", no_argument, NULL, 'c'},
   {"can-change", no_argument, NULL, 'C'},
+  {"logonserver", required_argument, NULL, 'd'},
   {"never-expires", no_argument, NULL, 'e'},
   {"expires", no_argument, NULL, 'E'},
   {"help", no_argument, NULL, 'h' },
@@ -49,10 +53,11 @@ static struct option longopts[] =
   {"maxage", required_argument, NULL, 'x'},
   {"length", required_argument, NULL, 'L'},
   {"status", no_argument, NULL, 'S'},
+  { "reg-store-pwd", no_argument, NULL, 'R'},
   {NULL, 0, NULL, 0}
 };
 
-static char opts[] = "cCeEhi:ln:pPuvx:L:S";
+static char opts[] = "cCd:eEhi:ln:pPuvx:L:SR";
 
 int
 eprint (int with_name, const char *fmt, ...)
@@ -108,10 +113,10 @@ EvalRet (int ret, const char *user)
 }
 
 PUSER_INFO_3
-GetPW (char *user, int print_win_name)
+GetPW (char *user, int print_win_name, LPCWSTR server)
 {
   char usr_buf[UNLEN + 1];
-  WCHAR name[2 * (UNLEN + 1)];
+  WCHAR name[UNLEN + 1];
   DWORD ret;
   PUSER_INFO_3 ui;
   struct passwd *pw;
@@ -132,30 +137,31 @@ GetPW (char *user, int print_win_name)
 	    }
 	}
     }
-  MultiByteToWideChar (CP_ACP, 0, user, -1, name, 2 * (UNLEN + 1));
-  ret = NetUserGetInfo (NULL, name, 3, (void *) &ui);
+  mbstowcs (name, user, UNLEN + 1);
+  ret = NetUserGetInfo (server, name, 3, (void *) &ui);
   return EvalRet (ret, user) ? NULL : ui;
 }
 
 int
-ChangePW (const char *user, const char *oldpwd, const char *pwd, int justcheck)
+ChangePW (const char *user, const char *oldpwd, const char *pwd, int justcheck,
+	  LPCWSTR server)
 {
-  WCHAR name[2 * (UNLEN + 1)], oldpass[512], pass[512];
+  WCHAR name[UNLEN + 1], oldpass[512], pass[512];
   DWORD ret;
 
-  MultiByteToWideChar (CP_ACP, 0, user, -1, name, 2 * (UNLEN + 1));
-  MultiByteToWideChar (CP_ACP, 0, pwd, -1, pass, 512);
+  mbstowcs (name, user, UNLEN + 1);
+  mbstowcs (pass, pwd, 512);
   if (! oldpwd)
     {
       USER_INFO_1003 ui;
 
       ui.usri1003_password = pass;
-      ret = NetUserSetInfo (NULL, name, 1003, (LPBYTE) &ui, NULL);
+      ret = NetUserSetInfo (server, name, 1003, (LPBYTE) &ui, NULL);
     }
   else
     {
-      MultiByteToWideChar (CP_ACP, 0, oldpwd, -1, oldpass, 512);
-      ret = NetUserChangePassword (NULL, name, oldpass, pass);
+      mbstowcs (oldpass, oldpwd, 512);
+      ret = NetUserChangePassword (server, name, oldpass, pass);
     }
   if (justcheck && ret != ERROR_INVALID_PASSWORD)
     return 0;
@@ -167,7 +173,7 @@ ChangePW (const char *user, const char *oldpwd, const char *pwd, int justcheck)
 }
 
 void
-PrintPW (PUSER_INFO_3 ui)
+PrintPW (PUSER_INFO_3 ui, LPCWSTR server)
 {
   time_t t = time (NULL) - ui->usri3_password_age;
   int ret;
@@ -184,7 +190,7 @@ PrintPW (PUSER_INFO_3 ui)
   printf ("Password expired           : %s",
 	(ui->usri3_password_expired) ? "yes\n" : "no\n");
   printf ("Latest password change     : %s", ctime(&t));
-  ret = NetUserModalsGet (NULL, 0, (void *) &mi);
+  ret = NetUserModalsGet (server, 0, (void *) &mi);
   if (! ret)
     {
       if (mi->usrmod0_max_passwd_age == TIMEQ_FOREVER)
@@ -208,12 +214,12 @@ PrintPW (PUSER_INFO_3 ui)
 }
 
 int
-SetModals (int xarg, int narg, int iarg, int Larg)
+SetModals (int xarg, int narg, int iarg, int Larg, LPCWSTR server)
 {
   int ret;
   PUSER_MODALS_INFO_0 mi;
 
-  ret = NetUserModalsGet (NULL, 0, (void *) &mi);
+  ret = NetUserModalsGet (server, 0, (void *) &mi);
   if (! ret)
     {
       if (xarg == 0)
@@ -237,7 +243,7 @@ SetModals (int xarg, int narg, int iarg, int Larg)
       if (Larg >= 0)
 	mi->usrmod0_min_passwd_len = Larg;
 
-      ret = NetUserModalsSet (NULL, 0, (LPBYTE) mi, NULL);
+      ret = NetUserModalsSet (server, 0, (LPBYTE) mi, NULL);
       NetApiBufferFree (mi);
     }
   return EvalRet (ret, NULL);
@@ -261,6 +267,9 @@ usage (FILE * stream, int status)
   "                           password aging rule.\n"
   "  -p, --pwd-not-required   no password required for USER.\n"
   "  -P, --pwd-required       password is required for USER.\n"
+  "  -R, --reg-store-pwd      enter password to store it in the registry for\n"
+  "                           later usage by services to be able to switch\n"
+  "                           to this user context with network credentials.\n"
   "\n"
   "System operations:\n"
   "  -i, --inactive NUM       set NUM of days before inactive accounts are disabled\n"
@@ -270,6 +279,10 @@ usage (FILE * stream, int status)
   "  -L, --length LEN         set system minimum password length to LEN.\n"
   "\n"
   "Other options:\n"
+  "  -d, --logonserver SERVER connect to SERVER (e.g. domain controller).\n"
+  "                           Default server is the local system, unless\n"
+  "                           changing the current user, in which case the\n"
+  "                           default is the content of $LOGONSERVER.\n"
   "  -S, --status             display password status for USER (locked, expired,\n"
   "                           etc.) plus global system password settings.\n"
   "  -h, --help               output usage information and exit.\n"
@@ -277,12 +290,64 @@ usage (FILE * stream, int status)
   "\n"
   "If no option is given, change USER's password.  If no user name is given,\n"
   "operate on current user.  System operations must not be mixed with user\n"
-  "operations.  Don't specify a USER when triggering a system operation. \n"
+  "operations.  Don't specify a USER when triggering a system operation.\n"
+  "\n"
+  "Don't specify a user or any other option together with the -R option.\n"
+  "Non-Admin users can only store their password if cygserver is running\n"
+  "as service under the SYSTEM account.\n"
+  "Note that storing even obfuscated passwords in the registry is not overly\n"
+  "secure.  Use this feature only if the machine is adequately locked down.\n"
+  "Don't use this feature if you don't need network access within a remote\n"
+  "session.  You can delete your stored password by using `passwd -R' and\n"
+  "specifying an empty password.\n"
   "\n"
   "Report bugs to <cygwin@cygwin.com>\n", prog_name);
   exit (status);
 }
 
+static int
+caller_is_admin ()
+{
+  static int is_admin = -1;
+  HANDLE token;
+  DWORD size;
+  PTOKEN_GROUPS grps;
+  SID_IDENTIFIER_AUTHORITY nt_auth = {SECURITY_NT_AUTHORITY};
+  PSID admin_grp; 
+  DWORD i;
+
+  if (is_admin == -1)
+    {
+      is_admin = 0;
+      if (OpenProcessToken (GetCurrentProcess (), TOKEN_READ, &token))
+	{
+	  GetTokenInformation (token, TokenGroups, NULL, 0, &size);
+	  grps = (PTOKEN_GROUPS) alloca (size);
+	  if (!GetTokenInformation(token, TokenGroups, grps, size, &size)
+	      || !AllocateAndInitializeSid (&nt_auth, 2,
+					    SECURITY_BUILTIN_DOMAIN_RID,
+					    DOMAIN_ALIAS_RID_ADMINS,
+					    0, 0, 0, 0, 0, 0, &admin_grp))
+	    is_admin = 0;
+	  else
+	    {
+	      for (i = 0; i < grps->GroupCount; ++i)
+		if (EqualSid (admin_grp, grps->Groups[i].Sid)
+		    && (grps->Groups[i].Attributes
+			& (SE_GROUP_ENABLED | SE_GROUP_USE_FOR_DENY_ONLY))
+		       == SE_GROUP_ENABLED)
+		  {
+		    is_admin = 1;
+		    break;
+		  }
+	      FreeSid (admin_grp);
+	    }
+	  CloseHandle (token);
+	}
+    }
+  return is_admin;
+}
+
 static void
 print_version ()
 {
@@ -309,8 +374,8 @@ Compiled on %s\n\
 int
 main (int argc, char **argv)
 {
-  char *c;
-  char user[64], oldpwd[64], newpwd[64];
+  char *c, *logonserver;
+  char user[UNLEN + 1], oldpwd[_PASSWORD_LEN + 1], newpwd[_PASSWORD_LEN + 1];
   int ret = 0;
   int cnt = 0;
   int opt;
@@ -327,7 +392,10 @@ main (int argc, char **argv)
   int popt = 0;
   int Popt = 0;
   int Sopt = 0;
-  PUSER_INFO_3 ui, li;
+  int Ropt = 0;
+  PUSER_INFO_3 ui;
+  int myself = 0;
+  LPWSTR server = NULL;
 
   prog_name = strrchr (argv[0], '/');
   if (prog_name == NULL)
@@ -340,6 +408,10 @@ main (int argc, char **argv)
   if (c)
     *c = '\0';
 
+  /* Use locale from environment.  If not set or set to "C", use UTF-8. */
+  setlocale (LC_CTYPE, "");
+  if (!strcmp (setlocale (LC_CTYPE, NULL), "C"))
+    setlocale (LC_CTYPE, "en_US.UTF-8");
   while ((opt = getopt_long (argc, argv, opts, longopts, NULL)) != EOF)
     switch (opt)
       {
@@ -348,20 +420,20 @@ main (int argc, char **argv)
         break;
 
       case 'i':
-	if (lopt || uopt || copt || Copt || eopt || Eopt || popt || Popt || Sopt)
+	if (lopt || uopt || copt || Copt || eopt || Eopt || popt || Popt || Sopt || Ropt)
 	  usage (stderr, 1);
 	if ((iarg = atoi (optarg)) < 0 || iarg > 999)
 	  return eprint (1, "Force logout time must be between 0 and 999.");
         break;
 
       case 'l':
-	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || uopt || Sopt)
+	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || uopt || Sopt || Ropt)
 	  usage (stderr, 1);
 	lopt = 1;
         break;
 
       case 'n':
-	if (lopt || uopt || copt || Copt || eopt || Eopt || popt || Popt || Sopt)
+	if (lopt || uopt || copt || Copt || eopt || Eopt || popt || Popt || Sopt || Ropt)
 	  usage (stderr, 1);
 	if ((narg = atoi (optarg)) < 0 || narg > 999)
 	  return eprint (1, "Minimum password age must be between 0 and 999.");
@@ -371,43 +443,59 @@ main (int argc, char **argv)
         break;
 
       case 'u':
-	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || lopt || Sopt)
+	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || lopt || Sopt || Ropt)
 	  usage (stderr, 1);
 	uopt = 1;
         break;
 
       case 'c':
-	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt)
+	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt || Ropt)
 	  usage (stderr, 1);
 	copt = 1;
         break;
 
       case 'C':
-	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt)
+	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt || Ropt)
 	  usage (stderr, 1);
 	Copt = 1;
         break;
 
+      case 'd':
+        {
+	  if (Ropt)
+	    usage (stderr, 1);
+	  char *tmpbuf = alloca (strlen (optarg) + 3);
+	  tmpbuf[0] = '\0';
+	  if (*optarg != '\\')
+	    strcpy (tmpbuf, "\\\\");
+	  strcat (tmpbuf, optarg);
+	  size_t len = mbstowcs (NULL, tmpbuf, 0);
+	  if (len > 0 && len != (size_t) -1)
+	    mbstowcs (server = alloca ((len + 1) * sizeof (wchar_t)),
+		      tmpbuf, len + 1);
+	}
+	break;
+
       case 'e':
-	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt)
+	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt || Ropt)
 	  usage (stderr, 1);
 	eopt = 1;
         break;
 
       case 'E':
-	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt)
+	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt || Ropt)
 	  usage (stderr, 1);
 	Eopt = 1;
         break;
 
       case 'p':
-	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt)
+	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt || Ropt)
 	  usage (stderr, 1);
 	popt = 1;
         break;
 
       case 'P':
-	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt)
+	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || Sopt || Ropt)
 	  usage (stderr, 1);
 	Popt = 1;
         break;
@@ -418,7 +506,7 @@ main (int argc, char **argv)
         break;
 
       case 'x':
-	if (lopt || uopt || copt || Copt || eopt || Eopt || popt || Popt || Sopt)
+	if (lopt || uopt || copt || Copt || eopt || Eopt || popt || Popt || Sopt || Ropt)
 	  usage (stderr, 1);
 	if ((xarg = atoi (optarg)) < 0 || xarg > 999)
 	  return eprint (1, "Maximum password age must be between 0 and 999.");
@@ -428,7 +516,7 @@ main (int argc, char **argv)
         break;
 
       case 'L':
-	if (lopt || uopt || copt || Copt || eopt || Eopt || popt || Popt || Sopt)
+	if (lopt || uopt || copt || Copt || eopt || Eopt || popt || Popt || Sopt || Ropt)
 	  usage (stderr, 1);
 	if ((Larg = atoi (optarg)) < 0 || Larg > LM20_PWLEN)
 	  return eprint (1, "Minimum password length must be between "
@@ -437,28 +525,69 @@ main (int argc, char **argv)
 
       case 'S':
 	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || lopt || uopt
-	    || copt || Copt || eopt || Eopt || popt || Popt)
+	    || copt || Copt || eopt || Eopt || popt || Popt || Ropt)
 	  usage (stderr, 1);
 	Sopt = 1;
         break;
 
+      case 'R':
+	if (xarg >= 0 || narg >= 0 || iarg >= 0 || Larg >= 0 || lopt || uopt
+	    || copt || Copt || eopt || Eopt || popt || Popt || Sopt
+	    || server)
+	  usage (stderr, 1);
+	Ropt = 1;
+      	break;
+
       default:
         usage (stderr, 1);
       }
+
+  if (Ropt)
+    {
+      if (optind < argc)
+        usage (stderr, 1);
+      printf (
+"This functionality stores a password in the registry for usage by services\n"
+"which need to change the user context and require network access.  Typical\n"
+"applications are interactive remote logons using sshd, cron task, etc.\n"
+"This password will always tried first when any privileged application is\n"
+"about to switch the user context.\n\n"
+"Note that storing even obfuscated passwords in the registry is not overly\n"
+"secure.  Use this feature only if the machine is adequately locked down.\n"
+"Don't use this feature if you don't need network access within a remote\n"
+"session.\n\n"
+"You can delete your stored password by specifying an empty password.\n\n");
+      strcpy (newpwd, getpass ("Enter your current password: "));
+      if (strcmp (newpwd, getpass ("Re-enter your current password: ")))
+        eprint (0, "Password is not identical.");
+      else if (cygwin_internal (CW_SET_PRIV_KEY, newpwd))
+	return eprint (0, "Storing password failed: %s", strerror (errno));
+      return 0;
+    }
+
   if (Larg >= 0 || xarg >= 0 || narg >= 0 || iarg >= 0)
     {
       if (optind < argc)
         usage (stderr, 1);
-      return SetModals (xarg, narg, iarg, Larg);
+      return SetModals (xarg, narg, iarg, Larg, server);
     }
 
   strcpy (user, optind >= argc ? getlogin () : argv[optind]);
 
-  li = GetPW (getlogin (), 0);
-  if (! li)
-    return 1;
+  /* Changing password for calling user?  Use logonserver for user as well. */
+  if (!server && optind >= argc)
+    {
+      myself = 1;
+      if ((logonserver = getenv ("LOGONSERVER")))
+	{
+	  size_t len = mbstowcs (NULL, logonserver, 0);
+	  if (len > 0 && len != (size_t) -1)
+	    mbstowcs (server = alloca ((len + 1) * sizeof (wchar_t)),
+		      logonserver, len + 1);
+	}
+    }
 
-  ui = GetPW (user, 1);
+  ui = GetPW (user, 1, server);
   if (! ui)
     return 1;
 
@@ -466,8 +595,6 @@ main (int argc, char **argv)
     {
       USER_INFO_1008 uif;
 
-      if (li->usri3_priv != USER_PRIV_ADMIN)
-        return eprint (0, "You have no maintenance privileges.");
       uif.usri1008_flags = ui->usri3_flags;
       if (lopt)
         {
@@ -492,26 +619,26 @@ main (int argc, char **argv)
 
       if (lopt || uopt || copt || Copt || eopt || Eopt || popt || Popt)
 	{
-          ret = NetUserSetInfo (NULL, ui->usri3_name, 1008, (LPBYTE) &uif,
+          ret = NetUserSetInfo (server, ui->usri3_name, 1008, (LPBYTE) &uif,
 	  			NULL);
           return EvalRet (ret, NULL);
 	}
       // Sopt
-      PrintPW (ui);
+      PrintPW (ui, server);
       return 0;
     }
 
-  if (li->usri3_priv != USER_PRIV_ADMIN && strcmp (getlogin (), user))
+  if (!caller_is_admin () && !myself)
     return eprint (0, "You may not change the password for %s.", user);
 
   eprint (0, "Enter the new password (minimum of 5, maximum of 8 characters).");
   eprint (0, "Please use a combination of upper and lower case letters and numbers.");
 
   oldpwd[0] = '\0';
-  if (li->usri3_priv != USER_PRIV_ADMIN)
+  if (!caller_is_admin ())
     {
       strcpy (oldpwd, getpass ("Old password: "));
-      if (ChangePW (user, oldpwd, oldpwd, 1))
+      if (ChangePW (user, oldpwd, oldpwd, 1, server))
         return 1;
     }
 
@@ -520,7 +647,7 @@ main (int argc, char **argv)
       strcpy (newpwd, getpass ("New password: "));
       if (strcmp (newpwd, getpass ("Re-enter new password: ")))
         eprint (0, "Password is not identical.");
-      else if (! ChangePW (user, *oldpwd ? oldpwd : NULL, newpwd, 0))
+      else if (! ChangePW (user, *oldpwd ? oldpwd : NULL, newpwd, 0, server))
         ret = 1;
       if (! ret && cnt < 2)
         eprint (0, "Try again.");
