@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2001, 2002 Red Hat, Inc.
+ * Copyright (c) 2000, 2001, 2002, 2009 Red Hat, Inc.
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include <windows.h>
 #include <getopt.h>
 
-static const char version[] = "$Revision: 1.9 $";
+static const char version[] = "$Revision: 1.12 $";
 static char *prog_name;
 
 static struct option longopts[] =
@@ -161,10 +161,12 @@ static HANDLE
 lookup_thread_id (DWORD threadId, int *tix)
 {
   int i;
+  *tix = 0;
   for (i=0; i<num_active_threads; i++)
     if (active_thread_ids[i] == threadId)
       {
-	if (tix) *tix = i;
+	if (tix)
+	  *tix = i;
 	return active_threads[i];
       }
   return 0;
@@ -594,7 +596,7 @@ run_program (char *cmdline)
 			{
 			  dll_ptr = cp+1;
 			}
-		      *cp = tolower (*cp);
+		      *cp = tolower ((unsigned char) *cp);
 		    }
 		}
 	    }
@@ -900,7 +902,7 @@ main (int argc, char **argv)
   hits = (HISTCOUNTER *)malloc (high_pc-low_pc+4);
   memset (hits, 0, high_pc-low_pc+4);
 
-  fprintf (stderr, "prun: [%08x,%08x] Running `%s'\n",
+  fprintf (stderr, "prun: [%08x,%08x] Running '%s'\n",
 	  low_pc, high_pc, argv[optind]);
 
   run_program (argv[optind]);
