@@ -1,6 +1,7 @@
 /* strace.cc
 
-   Copyright 2000, 2001, 2002, 2003, 2004 Red Hat Inc.
+   Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+   2009, 2010 Red Hat Inc.
 
    Written by Chris Faylor <cgf@redhat.com>
 
@@ -35,12 +36,17 @@ static const char version[] = "$Revision: 1.21 $";
 static const char *pgm;
 static int forkdebug = 1;
 static int numerror = 1;
-static int usecs = 1;
+static int show_usecs = 1;
 static int delta = 1;
-static int hhmmss = 0;
-static int bufsize = 0;
-static int new_window = 0;
-static long flush_period = 0;
+static int hhmmss;
+static int bufsize;
+static int new_window;
+static long flush_period;
+static int include_hex;
+static int quiet = -1;
+
+static unsigned char strace_active = 1;
+static int processes;
 
 static BOOL close_handle (HANDLE h, DWORD ok);
 
@@ -103,18 +109,6 @@ error (int geterrno, const char *fmt, ...)
 DWORD lastid = 0;
 HANDLE lasth;
 
-#define PROCFLAGS \
- PROCESS_ALL_ACCESS		/*(PROCESS_DUP_HANDLE | PROCESS_TERMINATE | PROCESS_VM_READ | PROCESS_VM_WRITE) */
-static void
-add_child (DWORD id, HANDLE hproc)
-{
-  child_list *c = children.next;
-  children.next = (child_list *) calloc (1, sizeof (child_list));
-  children.next->next = c;
-  lastid = children.next->id = id;
-  lasth = children.next->hproc = hproc;
-}
-
 static child_list *
 get_child (DWORD id)
 {
@@ -123,7 +117,23 @@ get_child (DWORD id)
     if (c->id == id)
       return c;
 
-  error (0, "no process id %d found", id);
+  return NULL;
+}
+
+static void
+add_child (DWORD id, HANDLE hproc)
+{
+  if (!get_child (id))
+    {
+      child_list *c = children.next;
+      children.next = (child_list *) calloc (1, sizeof (child_list));
+      children.next->next = c;
+      lastid = children.next->id = id;
+      lasth = children.next->hproc = hproc;
+      processes++;
+      if (!quiet)
+	fprintf (stderr, "Windows process %lu attached\n", id);
+    }
 }
 
 static void
@@ -138,6 +148,9 @@ remove_child (DWORD id)
 	child_list *c1 = c->next;
 	c->next = c1->next;
 	free (c1);
+	if (!quiet)
+	  fprintf (stderr, "Windows process %lu detached\n", id);
+	processes--;
 	return;
       }
 
@@ -249,7 +262,10 @@ ctrl_c (DWORD)
   return TRUE;
 }
 
-DWORD (*cygwin_internal) (int, ...);
+extern "C" {
+unsigned long (*cygwin_internal) (int, ...);
+WCHAR cygwin_dll_path[32768];
+};
 
 static int
 load_cygwin ()
@@ -267,6 +283,7 @@ load_cygwin ()
       errno = ENOENT;
       return 0;
     }
+  GetModuleFileNameW (h, cygwin_dll_path, 32768);
   if (!(cygwin_internal = (DWORD (*) (int, ...)) GetProcAddress (h, "cygwin_internal")))
     {
       errno = ENOSYS;
@@ -278,18 +295,13 @@ load_cygwin ()
 static void
 attach_process (pid_t pid)
 {
-  load_cygwin ();
   child_pid = (DWORD) cygwin_internal (CW_CYGWIN_PID_TO_WINPID, pid);
   if (!child_pid)
-    {
-      warn (0, "no such cygwin pid - %d", pid);
-      child_pid = pid;
-    }
+    child_pid = pid;
 
   if (!DebugActiveProcess (child_pid))
-    error (0, "couldn't attach to pid %d<%d> for debugging", pid, child_pid);
+    error (0, "couldn't attach to pid %d for debugging", child_pid);
 
-  printf ("Attached to pid %d (windows pid %u)\n", pid, (unsigned) child_pid);
   return;
 }
 
@@ -304,7 +316,8 @@ create_child (char **argv)
   BOOL ret;
   DWORD flags;
 
-  *argv = cygpath (*argv, NULL);
+  if (strchr (*argv, '/'))
+      *argv = cygpath (*argv, NULL);
   memset (&si, 0, sizeof (si));
   si.cb = sizeof (si);
 
@@ -316,12 +329,22 @@ create_child (char **argv)
   make_command_line (one_line, argv);
 
   SetConsoleCtrlHandler (NULL, 0);
+  const char *cygwin_env = getenv ("CYGWIN");
+  const char *space;
+  if (cygwin_env)
+    space = " ";
+  else
+    space = cygwin_env = "";
+  char *newenv = (char *) malloc (sizeof ("CYGWIN=noglob") + strlen (space) + strlen (cygwin_env));
+  sprintf (newenv, "CYGWIN=noglob%s%s", space, cygwin_env);
+  _putenv (newenv);
   ret = CreateProcess (0, one_line.buf,	/* command line */
 		       NULL,	/* Security */
 		       NULL,	/* thread */
 		       TRUE,	/* inherit handles */
 		       flags,	/* start flags */
-		       NULL, NULL,	/* current directory */
+		       NULL,	/* default environment */
+		       NULL,	/* current directory */
 		       &si, &pi);
   if (!ret)
     error (0, "error creating process %s, (error %d)", *argv,
@@ -379,7 +402,7 @@ syst (long long t)
 {
   FILETIME n;
   static SYSTEMTIME st;
-  long long now = t + ((long long) usecs * 10);
+  long long now = t /*+ ((long long) usecs * 10)*/;
   n.dwHighDateTime = now >> 32;
   n.dwLowDateTime = now & 0xffffffff;
   FileTimeToSystemTime (&n, &st);
@@ -394,6 +417,8 @@ handle_output_debug_string (DWORD id, LPVOID p, unsigned mask, FILE *ofile)
   char alen[3 + 8 + 1];
   DWORD nbytes;
   child_list *child = get_child (id);
+  if (!child)
+    error (0, "no process id %d found", id);
   HANDLE hchild = child->hproc;
 #define INTROLEN (sizeof (alen) - 1)
 
@@ -421,12 +446,12 @@ handle_output_debug_string (DWORD id, LPVOID p, unsigned mask, FILE *ofile)
   else
     {
       special = len;
-      if (special == _STRACE_INTERFACE_ACTIVATE_ADDR)
+      if (special == _STRACE_INTERFACE_ACTIVATE_ADDR || special == _STRACE_CHILD_PID)
 	len = 17;
     }
 
   char *buf;
-  buf = (char *) alloca (len + 65) + 10;
+  buf = (char *) alloca (len + 85) + 20;
 
   if (!ReadProcessMemory (hchild, ((char *) p) + INTROLEN, buf, len, &nbytes))
     error (0, "couldn't get message from subprocess, windows error %d",
@@ -435,18 +460,24 @@ handle_output_debug_string (DWORD id, LPVOID p, unsigned mask, FILE *ofile)
   buf[len] = '\0';
   char *s = strtok (buf, " ");
 
-  unsigned n = strtoul (s, NULL, 16);
+  unsigned long n = strtoul (s, NULL, 16);
 
   s = strchr (s, '\0') + 1;
 
+  if (special == _STRACE_CHILD_PID)
+    {
+      if (!DebugActiveProcess (n))
+	error (0, "couldn't attach to subprocess %d for debugging, "
+	       "windows error %d", n, GetLastError ());
+      return;
+    }
+
   if (special == _STRACE_INTERFACE_ACTIVATE_ADDR)
     {
-      DWORD new_flag = 1;
-      if (!WriteProcessMemory (hchild, (LPVOID) n, &new_flag,
-			       sizeof (new_flag), &nbytes))
-	error (0,
-	       "couldn't write strace flag to subprocess, windows error %d",
-	       GetLastError ());
+      if (!WriteProcessMemory (hchild, (LPVOID) n, &strace_active,
+			       sizeof (strace_active), &nbytes))
+	error (0, "couldn't write strace flag to subprocess at %p, "
+	       "windows error %d", n, GetLastError ());
       return;
     }
 
@@ -473,7 +504,7 @@ handle_output_debug_string (DWORD id, LPVOID p, unsigned mask, FILE *ofile)
   else
     {
       ptrest = q;
-      ptusec = s;
+      ptusec = show_usecs ? s : ptrest;
       usecs = dusecs;
     }
 
@@ -558,6 +589,12 @@ handle_output_debug_string (DWORD id, LPVOID p, unsigned mask, FILE *ofile)
       memcpy ((s -= len), intbuf, len);
     }
 
+  if (include_hex)
+    {
+      s -= 8;
+      sprintf (s, "%p", (void *) n);
+      strchr (s, '\0')[0] = ' ';
+    }
   child->last_usecs = usecs;
   if (numerror || !output_winerror (ofile, s))
     fputs (s, ofile);
@@ -565,11 +602,11 @@ handle_output_debug_string (DWORD id, LPVOID p, unsigned mask, FILE *ofile)
     fflush (ofile);
 }
 
-static void
+static DWORD
 proc_child (unsigned mask, FILE *ofile, pid_t pid)
 {
+  DWORD res = 0;
   DEBUG_EVENT ev;
-  int processes = 0;
   time_t cur_time, last_time;
 
   SetThreadPriority (GetCurrentThread (), THREAD_PRIORITY_HIGHEST);
@@ -601,7 +638,6 @@ proc_child (unsigned mask, FILE *ofile, pid_t pid)
 	  if (ev.u.CreateProcessInfo.hFile)
 	    CloseHandle (ev.u.CreateProcessInfo.hFile);
 	  add_child (ev.dwProcessId, ev.u.CreateProcessInfo.hProcess);
-	  processes++;
 	  break;
 
 	case CREATE_THREAD_DEBUG_EVENT:
@@ -619,33 +655,33 @@ proc_child (unsigned mask, FILE *ofile, pid_t pid)
 	  break;
 
 	case EXIT_PROCESS_DEBUG_EVENT:
+	  res = ev.u.ExitProcess.dwExitCode >> 8;
 	  remove_child (ev.dwProcessId);
 	  break;
 	case EXCEPTION_DEBUG_EVENT:
-	  if (ev.u.Exception.ExceptionRecord.ExceptionCode !=
-	      STATUS_BREAKPOINT)
+	  if (ev.u.Exception.ExceptionRecord.ExceptionCode != STATUS_BREAKPOINT)
 	    {
 	      status = DBG_EXCEPTION_NOT_HANDLED;
-#if 0
-	      fprintf (stderr, "exception %p at %p\n",
-		       ev.u.Exception.ExceptionRecord.ExceptionCode,
-		       ev.u.Exception.ExceptionRecord.ExceptionAddress);
-#endif
+	      if (ev.u.Exception.dwFirstChance)
+		fprintf (ofile, "--- Process %lu, exception %p at %p\n", ev.dwProcessId,
+			 (void *) ev.u.Exception.ExceptionRecord.ExceptionCode,
+			 ev.u.Exception.ExceptionRecord.ExceptionAddress);
 	    }
 	  break;
 	}
       if (!ContinueDebugEvent (ev.dwProcessId, ev.dwThreadId, status))
 	error (0, "couldn't continue debug event, windows error %d",
 	       GetLastError ());
-      if (ev.dwDebugEventCode == EXIT_PROCESS_DEBUG_EVENT && --processes == 0)
+      if (!processes)
 	break;
     }
+
+  return res;
 }
 
 static void
 dotoggle (pid_t pid)
 {
-  load_cygwin ();
   child_pid = (DWORD) cygwin_internal (CW_CYGWIN_PID_TO_WINPID, pid);
   if (!child_pid)
     {
@@ -658,16 +694,15 @@ dotoggle (pid_t pid)
   return;
 }
 
-static void
+static DWORD
 dostrace (unsigned mask, FILE *ofile, pid_t pid, char **argv)
 {
   if (!pid)
     create_child (argv);
   else
     attach_process (pid);
-  proc_child (mask, ofile, pid);
 
-  return;
+  return proc_child (mask, ofile, pid);
 }
 
 typedef struct tag_mask_mnemonic
@@ -696,6 +731,8 @@ static const mask_mnemonic mnemonic_table[] = {
   {_STRACE_NOMUTEX, "nomutex"},
   {_STRACE_MALLOC, "malloc"},
   {_STRACE_THREAD, "thread"},
+  {_STRACE_PTHREAD, "pthread"},
+  {_STRACE_SPECIAL, "special"},
   {0, NULL}
 };
 
@@ -805,11 +842,13 @@ Trace system calls and signals\n\
                                numbers for Windows errors\n\
   -o, --output=FILENAME        set output file to FILENAME\n\
   -p, --pid=n                  attach to executing program with cygwin pid n\n\
+  -q, --quiet                  suppress messages about attaching, detaching, etc.\n\
   -S, --flush-period=PERIOD    flush buffered strace output every PERIOD secs\n\
   -t, --timestamp              use an absolute hh:mm:ss timestamp insted of \n\
                                the default microsecond timestamp.  Implies -d\n\
   -T, --toggle                 toggle tracing in a process already being\n\
                                traced. Requires -p <pid>\n\
+  -u, --usecs                  toggle printing of microseconds timestamp\n\
   -v, --version                output version information and exit\n\
   -w, --new-window             spawn program under test in a new window\n\
 \n", pgm, pgm);
@@ -822,24 +861,27 @@ Trace system calls and signals\n\
 \n\
     Mnemonic Hex     Corresponding Def  Description\n\
     =========================================================================\n\
-    all      0x00001 (_STRACE_ALL)      All strace messages.\n\
-    flush    0x00002 (_STRACE_FLUSH)    Flush output buffer after each message.\n\
-    inherit  0x00004 (_STRACE_INHERIT)  Children inherit mask from parent.\n\
-    uhoh     0x00008 (_STRACE_UHOH)     Unusual or weird phenomenon.\n\
-    syscall  0x00010 (_STRACE_SYSCALL)  System calls.\n\
-    startup  0x00020 (_STRACE_STARTUP)  argc/envp printout at startup.\n\
-    debug    0x00040 (_STRACE_DEBUG)    Info to help debugging. \n\
-    paranoid 0x00080 (_STRACE_PARANOID) Paranoid info.\n\
-    termios  0x00100 (_STRACE_TERMIOS)  Info for debugging termios stuff.\n\
-    select   0x00200 (_STRACE_SELECT)   Info on ugly select internals.\n\
-    wm       0x00400 (_STRACE_WM)       Trace Windows msgs (enable _strace_wm).\n\
-    sigp     0x00800 (_STRACE_SIGP)     Trace signal and process handling.\n\
-    minimal  0x01000 (_STRACE_MINIMAL)  Very minimal strace output.\n\
-    exitdump 0x04000 (_STRACE_EXITDUMP) Dump strace cache on exit.\n\
-    system   0x08000 (_STRACE_SYSTEM)   Serious error; goes to console and log.\n\
-    nomutex  0x10000 (_STRACE_NOMUTEX)  Don't use mutex for synchronization.\n\
-    malloc   0x20000 (_STRACE_MALLOC)   Trace malloc calls.\n\
-    thread   0x40000 (_STRACE_THREAD)   Thread-locking calls.\n\
+    all      0x000001 (_STRACE_ALL)      All strace messages.\n\
+    flush    0x000002 (_STRACE_FLUSH)    Flush output buffer after each message.\n\
+    inherit  0x000004 (_STRACE_INHERIT)  Children inherit mask from parent.\n\
+    uhoh     0x000008 (_STRACE_UHOH)     Unusual or weird phenomenon.\n\
+    syscall  0x000010 (_STRACE_SYSCALL)  System calls.\n\
+    startup  0x000020 (_STRACE_STARTUP)  argc/envp printout at startup.\n\
+    debug    0x000040 (_STRACE_DEBUG)    Info to help debugging. \n\
+    paranoid 0x000080 (_STRACE_PARANOID) Paranoid info.\n\
+    termios  0x000100 (_STRACE_TERMIOS)  Info for debugging termios stuff.\n\
+    select   0x000200 (_STRACE_SELECT)   Info on ugly select internals.\n\
+    wm       0x000400 (_STRACE_WM)       Trace Windows msgs (enable _strace_wm).\n\
+    sigp     0x000800 (_STRACE_SIGP)     Trace signal and process handling.\n\
+    minimal  0x001000 (_STRACE_MINIMAL)  Very minimal strace output.\n\
+    pthread  0x002000 (_STRACE_PTHREAD)	Pthread calls.\n\
+    exitdump 0x004000 (_STRACE_EXITDUMP) Dump strace cache on exit.\n\
+    system   0x008000 (_STRACE_SYSTEM)   Serious error; goes to console and log.\n\
+    nomutex  0x010000 (_STRACE_NOMUTEX)  Don't use mutex for synchronization.\n\
+    malloc   0x020000 (_STRACE_MALLOC)   Trace malloc calls.\n\
+    thread   0x040000 (_STRACE_THREAD)   Thread-locking calls.\n\
+    special  0x100000 (_STRACE_SPECIAL)  Special debugging printfs for\n\
+                                         non-checked-in code\n\
 ");
   if (where == stderr)
     fprintf (stderr, "Try '%s --help' for more information.\n", pgm);
@@ -850,11 +892,13 @@ struct option longopts[] = {
   {"buffer-size", required_argument, NULL, 'b'},
   {"help", no_argument, NULL, 'h'},
   {"flush-period", required_argument, NULL, 'S'},
+  {"hex", no_argument, NULL, 'H'},
   {"mask", required_argument, NULL, 'm'},
   {"new-window", no_argument, NULL, 'w'},
   {"output", required_argument, NULL, 'o'},
   {"no-delta", no_argument, NULL, 'd'},
   {"pid", required_argument, NULL, 'p'},
+  {"quiet", no_argument, NULL, 'q'},
   {"timestamp", no_argument, NULL, 't'},
   {"toggle", no_argument, NULL, 'T'},
   {"trace-children", no_argument, NULL, 'f'},
@@ -864,7 +908,7 @@ struct option longopts[] = {
   {NULL, 0, NULL, 0}
 };
 
-static const char *const opts = "+b:dhfm:no:p:S:tTuvw";
+static const char *const opts = "+b:dhHfm:no:p:qS:tTuvw";
 
 static void
 print_version ()
@@ -884,7 +928,7 @@ print_version ()
   printf ("\
 %s (cygwin) %.*s\n\
 System Trace\n\
-Copyright 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.\n\
+Copyright 2000, 2001, 2002, 2003, 2004, 2005 Red Hat, Inc.\n\
 Compiled on %s\n\
 ", pgm, len, v, __DATE__);
 }
@@ -897,6 +941,15 @@ main (int argc, char **argv)
   pid_t pid = 0;
   int opt;
   int toggle = 0;
+  int sawquiet = -1;
+
+  if (load_cygwin ())
+    {
+      char **av = (char **) cygwin_internal (CW_ARGV);
+      if (av && (DWORD) av != (DWORD) -1)
+	for (argc = 0, argv = av; *av; av++)
+	  argc++;
+    }
 
   if (!(pgm = strrchr (*argv, '\\')) && !(pgm = strrchr (*argv, '/')))
     pgm = *argv;
@@ -919,6 +972,9 @@ main (int argc, char **argv)
 	// Print help and exit
 	usage (stdout);
 	break;
+      case 'H':
+	include_hex ^= 1;
+	break;
       case 'm':
 	{
 	  char *endptr;
@@ -935,7 +991,7 @@ character #%d.\n", optarg, (int) (endptr - optarg), endptr);
 	numerror ^= 1;
 	break;
       case 'o':
-	if ((ofile = fopen (cygpath (optarg, NULL), "w")) == NULL)
+	if ((ofile = fopen (cygpath (optarg, NULL), "wb")) == NULL)
 	  error (1, "can't open %s", optarg);
 #ifdef F_SETFD
 	(void) fcntl (fileno (ofile), F_SETFD, 0);
@@ -943,6 +999,13 @@ character #%d.\n", optarg, (int) (endptr - optarg), endptr);
 	break;
       case 'p':
 	pid = strtoul (optarg, NULL, 10);
+	strace_active |= 2;
+	break;
+      case 'q':
+	if (sawquiet < 0)
+	  sawquiet = 1;
+	else
+	  sawquiet ^= 1;
 	break;
       case 'S':
 	flush_period = strtoul (optarg, NULL, 10);
@@ -955,7 +1018,8 @@ character #%d.\n", optarg, (int) (endptr - optarg), endptr);
 	break;
       case 'u':
 	// FIXME: currently unimplemented
-	usecs ^= 1;
+	show_usecs ^= 1;
+	delta ^= 1;
 	break;
       case 'v':
 	// Print version info and exit
@@ -978,6 +1042,13 @@ character #%d.\n", optarg, (int) (endptr - optarg), endptr);
   if (toggle && !pid)
     error (0, "must provide a process id to toggle tracing");
 
+  if (!pid)
+    quiet = sawquiet < 0 || !sawquiet;
+  else if (sawquiet < 0)
+    quiet = 0;
+  else
+    quiet = sawquiet;
+
   if (!mask)
     mask = _STRACE_ALL;
 
@@ -987,10 +1058,12 @@ character #%d.\n", optarg, (int) (endptr - optarg), endptr);
   if (!ofile)
     ofile = stdout;
 
+  DWORD res = 0;
   if (toggle)
     dotoggle (pid);
   else
-    dostrace (mask, ofile, pid, argv + optind);
+    res = dostrace (mask, ofile, pid, argv + optind);
+  return res;
 }
 
 #undef CloseHandle
